@@ -6,7 +6,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2018	PgPool Global Development Group
+ * Copyright (c) 2003-2020	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -83,7 +83,7 @@ pool_init_pool_passwd(char *pool_passwd_filename, POOL_PASSWD_MODE mode)
 		}
 		ereport(ERROR,
 				(errmsg("initializing pool password, failed to open file:\"%s\"", pool_passwd_filename),
-				 errdetail("file open failed with error:\"%s\"", strerror(errno))));
+				 errdetail("file open failed with error:\"%m\"")));
 	}
 }
 
@@ -329,7 +329,7 @@ userMatchesString(char *buf, char *user)
 }
 
 /*
- * user:passwod[:user:password]
+ * user:password[:user:password]
  */
 PasswordMapping *
 pool_get_user_credentials(char *username)
@@ -464,7 +464,7 @@ get_pgpool_config_user_password(char *username, char *password_in_config)
 	PasswordMapping *password_mapping = NULL;
 
 	/*
-	 * if the password specified in config is empty strin or NULL look for the
+	 * if the password specified in config is empty string or NULL look for the
 	 * password in pool_passwd file
 	 */
 	if (password_in_config == NULL || strlen(password_in_config) == 0)
@@ -621,14 +621,28 @@ read_pool_key(char *key_file_path)
 	if (strlen(key_file_path) == 0)
 		return NULL;
 
+	/*
+	 * To prevent file-swapping due to file race conditions,
+	 * we open the key file before checking it by stat().
+	 */
 	/* If password file cannot be opened, ignore it. */
-	if (stat(key_file_path, &stat_buf) != 0)
+	if ( (fp = fopen(key_file_path, "r")) == NULL)
 		return NULL;
 
+	if (fstat(fileno(fp), &stat_buf) != 0)
+	{
+		ereport(WARNING,
+				(errmsg("failed to stat pool key file"),
+				 errdetail("fstat failed with reason: \"%m\"")));
+		fclose(fp);
+		return NULL;
+	}
+
 	if (!S_ISREG(stat_buf.st_mode))
 	{
 		ereport(WARNING,
 				(errmsg("pool key file \"%s\" is not a plain file\n", key_file_path)));
+		fclose(fp);
 		return NULL;
 	}
 
@@ -639,13 +653,10 @@ read_pool_key(char *key_file_path)
 				(errmsg("pool key file \"%s\" has group or world access; permissions should be u=rw (0600) or less\n",
 						key_file_path)));
 		/* do we want to allow unsecure pool key file ? */
+		/* fclose(fp); */
 		/* return NULL; */
 	}
 
-	fp = fopen(key_file_path, "r");
-	if (fp == NULL)
-		return NULL;
-
 	while (!feof(fp) && !ferror(fp))
 	{
 		int			len;
@@ -669,3 +680,45 @@ read_pool_key(char *key_file_path)
 
 #undef LINELEN
 }
+
+/*
+ * Check password type is md5 hashed or not. recovery_password and
+ * wd_lifecheck_password are not allowed to be md5 hashed format.
+ * The kind of returns of this function is follow;
+ *  0: password is not md5 hashed
+ * -1: password is md5 hashed
+ * -2: password is not found
+ */
+int
+chceck_password_type_is_not_md5(char *username, char *password_in_config)
+{
+	PasswordType passwordType = PASSWORD_TYPE_UNKNOWN;
+
+	/*
+	 * if the password specified in config is empty string or NULL look for the
+	 * password in pool_passwd file
+	 */
+	if (password_in_config == NULL || strlen(password_in_config) == 0)
+	{
+		PasswordMapping *password_mapping = NULL;
+		password_mapping = pool_get_user_credentials(username);
+		if (password_mapping == NULL)
+		{
+			return -2;
+		}
+		passwordType = password_mapping->pgpoolUser.passwordType;
+		delete_passwordMapping(password_mapping);
+	}
+	else
+	{
+		passwordType = get_password_type(password_in_config);
+	}
+
+	/* if the password type is MD5 hash return -1*/
+	if (passwordType == PASSWORD_TYPE_MD5)
+	{
+		return -1;
+	}
+
+	return 0;
+}
\ No newline at end of file
