@@ -6,7 +6,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2019	PgPool Global Development Group
+ * Copyright (c) 2003-2020	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -41,7 +41,7 @@ typedef enum
 	POOL_UNKNOWN,				/* Unknown. Need to ask backend */
 	POOL_READ_UNCOMMITTED,		/* Read uncommitted */
 	POOL_READ_COMMITTED,		/* Read committed */
-	POOL_REPEATABLE_READ,		/* Rpeatable read */
+	POOL_REPEATABLE_READ,		/* Repeatable read */
 	POOL_SERIALIZABLE			/* Serializable */
 }			POOL_TRANSACTION_ISOLATION;
 
@@ -60,8 +60,8 @@ typedef enum
  */
 typedef enum
 {
-	POOL_SENT_MESSAGE_CREATED,	/* initial state of sent meesage */
-	POOL_SENT_MESSAGE_CLOSED	/* sent meesage closed but close complete
+	POOL_SENT_MESSAGE_CREATED,	/* initial state of sent message */
+	POOL_SENT_MESSAGE_CLOSED	/* sent message closed but close complete
 								 * message has not arrived yet */
 }			POOL_SENT_MESSAGE_STATE;
 
@@ -132,7 +132,7 @@ typedef struct
 								 * kind */
 	int			contents_len;	/* message packet length */
 	char		query[QUERY_STRING_BUFFER_LEN]; /* copy of original query */
-	char		statement[MAX_IDENTIFIER_LEN];	/* prepared statment name if
+	char		statement[MAX_IDENTIFIER_LEN];	/* prepared statement name if
 												 * any */
 	char		portal[MAX_IDENTIFIER_LEN]; /* portal name if any */
 	bool		is_rows_returned;	/* true if the message could produce row
@@ -156,6 +156,13 @@ typedef struct {
 	POOL_TEMP_TABLE_STATE	state;	/* see above */
 }			POOL_TEMP_TABLE;
 
+
+typedef enum
+{
+	SI_NO_SNAPSHOT,
+	SI_SNAPSHOT_PREPARED
+} SI_STATE;
+
 /*
  * Per session context:
  */
@@ -248,14 +255,14 @@ typedef struct
 								 * transaction */
 
 	/*
-	 * Parse/Bind/Decribe/Execute/Close message queue.
+	 * Parse/Bind/Describe/Execute/Close message queue.
 	 */
 	List	   *pending_messages;
 
 	/*
 	 * The last pending message. Reset at Ready for query.  Note that this is
 	 * a shallow copy of pending message.  Once the are is reset,
-	 * previos_message_exists is set to false.
+	 * previous_message_exists is set to false.
 	 */
 	bool		previous_message_exists;
 	POOL_PENDING_MESSAGE previous_message;
@@ -278,10 +285,23 @@ typedef struct
 	 */
 	List	   *temp_tables;
 
+	bool		is_in_transaction;
+
+	/*
+	 * Current transaction temp write list
+	 */
+	List	   *transaction_temp_write_list;
+
 #ifdef NOT_USED
-	/* Preferred "master" node id. Only used for SimpleForwardToFrontend. */
-	int			preferred_master_node_id;
+	/* Preferred "main" node id. Only used for SimpleForwardToFrontend. */
+	int			preferred_main_node_id;
 #endif
+
+	/* Whether snapshot is acquired in this transaction. Only used by Snapshot Isolation mode. */
+	SI_STATE	si_state;
+	/* Whether transaction is read only. Only used by Snapshot Isolation mode. */
+	SI_STATE	transaction_read_only;
+
 }			POOL_SESSION_CONTEXT;
 
 extern void pool_init_session_context(POOL_CONNECTION * frontend, POOL_CONNECTION_POOL * backend);
@@ -366,9 +386,9 @@ extern void	pool_temp_tables_remove_pending(void);
 extern void	pool_temp_tables_dump(void);
 
 #ifdef NOT_USED
-extern void pool_set_preferred_master_node_id(int node_id);
-extern int	pool_get_preferred_master_node_id(void);
-extern void pool_reset_preferred_master_node_id(void);
+extern void pool_set_preferred_main_node_id(int node_id);
+extern int	pool_get_preferred_main_node_id(void);
+extern void pool_reset_preferred_main_node_id(void);
 #endif
 
 #ifdef NOT_USED
