@@ -4,8 +4,8 @@
  *	  PostgreSQL's list of SQL keywords
  *
  *
- * Portions Copyright (c) 2003-2019, PgPool Global Development Group
- * Portions Copyright (c) 1996-2019, PostgreSQL Global Development Group
+ * Portions Copyright (c) 2003-2021, PgPool Global Development Group
+ * Portions Copyright (c) 1996-2021, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  * src/include/common/keywords.h
@@ -26,9 +26,11 @@
 #ifndef FRONTEND
 extern PGDLLIMPORT const ScanKeywordList ScanKeywords;
 extern PGDLLIMPORT const uint8 ScanKeywordCategories[];
+extern PGDLLIMPORT const bool ScanKeywordBareLabel[];
 #else
 extern const ScanKeywordList ScanKeywords;
 extern const uint8 ScanKeywordCategories[];
+extern const bool ScanKeywordBareLabel[];
 #endif
 
 #endif							/* KEYWORDS_H */
