@@ -42,7 +42,7 @@ typedef struct
 	bool		has_view;		/* True if view is used */
 	bool		has_function_call;	/* True if write function call is used */
 	int			pg_terminate_backend_pid;	/* pid argument of
-											 * pg_terminate_backedn_call(if
+											 * pg_terminate_backend_call(if
 											 * used) */
 	bool		has_non_immutable_function_call;	/* True if non immutable
 													 * functions are used */
@@ -70,6 +70,7 @@ extern int	pool_table_name_to_oid(char *table_name);
 extern int	pool_extract_table_oids_from_select_stmt(Node *node, SelectContext * ctx);
 extern RangeVar *makeRangeVarFromNameList(List *names);
 extern char *make_table_name_from_rangevar(RangeVar *rangevar);
+extern char *make_function_name_from_funccall(FuncCall *fcall);
 extern int	pattern_compare(char *str, const int type, const char *param_name);
 extern bool is_unlogged_table(char *table_name);
 extern bool is_view(char *table_name);
