@@ -5,7 +5,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2018	PgPool Global Development Group
+ * Copyright (c) 2003-2022	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -20,6 +20,9 @@
  */
 #include "pool.h"
 #include "pool_config.h"
+#include "version.h"
+#include "pool_config_variables.h"
+
 #include <fcntl.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -35,17 +38,20 @@
 #include <errno.h>
 #include <string.h>
 #include <libgen.h>
+
 #include "utils/elog.h"
 #include "utils/palloc.h"
 #include "utils/memutils.h"
 #include "utils/pool_path.h"
+#include "utils/pool_signal.h"
+#include "utils/pool_ipc.h"
+#include "utils/ps_status.h"
+#include "utils/pool_ssl.h"
 
-#include "version.h"
 #include "auth/pool_passwd.h"
 #include "auth/pool_hba.h"
 #include "query_cache/pool_memqcache.h"
 #include "watchdog/wd_utils.h"
-#include "pool_config_variables.h"
 
 
 static bool get_pool_key_filename(char *poolKeyFile);
@@ -71,6 +77,7 @@ int			myargc;
 char	  **myargv;
 int			assert_enabled = 0;
 char	   *pool_key = NULL;
+
 int
 main(int argc, char **argv)
 {
@@ -216,6 +223,8 @@ main(int argc, char **argv)
 	hba_file = make_absolute_path(hba_file_path, base_dir);
 
 	mypid = getpid();
+	SetProcessGlobalVariables(PT_MAIN);
+
 
 	pool_init_config();
 
@@ -250,7 +259,8 @@ main(int argc, char **argv)
 			{
 				ereport(FATAL,
 						(return_code(1),
-						 errmsg("could not reload configuration file pid: %d. reason: %s", pid, strerror(errno))));
+						 errmsg("could not reload configuration file pid: %d", pid),
+						 errdetail("%m")));
 			}
 			exit(0);
 		}
@@ -352,7 +362,7 @@ main(int argc, char **argv)
 
 	pool_semaphore_create(MAX_NUM_SEMAPHORES);
 
-	PgpoolMain(discard_status, clear_memcache_oidmaps); /* this is an infinate
+	PgpoolMain(discard_status, clear_memcache_oidmaps); /* this is an infinite
 														 * loop */
 
 	exit(0);
@@ -455,7 +465,7 @@ daemonize(void)
 	{
 		ereport(FATAL,
 				(errmsg("could not daemonize the pgpool-II"),
-				 errdetail("fork() system call failed with reason: \"%s\"", strerror(errno))));
+				 errdetail("fork() system call failed with reason: \"%m\"")));
 	}
 	else if (pid > 0)
 	{							/* parent */
@@ -467,23 +477,25 @@ daemonize(void)
 	{
 		ereport(FATAL,
 				(errmsg("could not daemonize the pgpool-II"),
-				 errdetail("setsid() system call failed with reason: \"%s\"", strerror(errno))));
+				 errdetail("setsid() system call failed with reason: \"%m\"")));
 	}
 #endif
 
 	mypid = getpid();
+	SetProcessGlobalVariables(PT_MAIN);
 	write_pid_file();
 	if (chdir("/"))
 		ereport(WARNING,
 				(errmsg("change directory failed"),
-				 errdetail("chdir() system call failed with reason: \"%s\"", strerror(errno))));
+				 errdetail("chdir() system call failed with reason: \"%m\"")));
 
 	/* redirect stdin, stdout and stderr to /dev/null */
 	i = open("/dev/null", O_RDWR);
 	if (i < 0)
 	{
 		ereport(WARNING,
-				(errmsg("failed to open \"/dev/null\", open() failed with error \"%s\"", strerror(errno))));
+				(errmsg("failed to open \"/dev/null\""),
+				 errdetail("%m")));
 	}
 	else
 	{
@@ -520,19 +532,31 @@ stop_me(void)
 				(errmsg("could not read pid file")));
 	}
 
-	if (kill(pid, stop_sig) == -1)
+	for (;;)
 	{
-		ereport(FATAL,
-				(errmsg("could not stop process with pid: %d", pid),
-				 errdetail("\"%s\"", strerror(errno))));
-	}
-	ereport(LOG,
-			(errmsg("stop request sent to pgpool. waiting for termination...")));
+		int		cnt = 5;	/* sending sinal retry interval */
 
-	while (kill(pid, 0) == 0)
-	{
-		fprintf(stderr, ".");
-		sleep(1);
+		if (kill(pid, stop_sig) == -1)
+		{
+			ereport(FATAL,
+					(errmsg("could not stop process with pid: %d", pid),
+					 errdetail("%m")));
+		}
+		ereport(LOG,
+				(errmsg("stop request sent to pgpool (pid: %d). waiting for termination...", pid)));
+
+		while (kill(pid, 0) == 0)
+		{
+			fprintf(stderr, ".");
+			sleep(1);
+			cnt--;
+			/* If pgpool did not stop within 5 seconds, break the loop and try
+			 * to send the signal again */
+			if (cnt <= 0)
+				break;
+		}
+		if (cnt > 0)
+			break;
 	}
 	fprintf(stderr, "done.\n");
 	pid_file = get_pid_file_path();
@@ -562,8 +586,9 @@ get_pid_file_path(void)
 		if (conf_dir == NULL)
 		{
 			ereport(LOG,
-					(errmsg("failed to get the dirname of pid file:\"%s\". reason: %s",
-							pool_config->pid_file_name, strerror(errno))));
+					(errmsg("failed to get the dirname of pid file:\"%s\"",
+							pool_config->pid_file_name),
+					 errdetail("%m")));
 			return NULL;
 		}
 		path_size = strlen(conf_dir) + strlen(pool_config->pid_file_name) + 1 + 1;
@@ -613,16 +638,18 @@ read_pid_file(void)
 		close(fd);
 		pfree(pid_file);
 		ereport(FATAL,
-				(errmsg("could not read pid file as \"%s\". reason: %s",
-						pool_config->pid_file_name, strerror(errno))));
+				(errmsg("could not read pid file \"%s\"",
+						pool_config->pid_file_name),
+				 errdetail("%m")));
 	}
 	else if (readlen == 0)
 	{
 		close(fd);
 		pfree(pid_file);
 		ereport(FATAL,
-				(errmsg("EOF detected while reading pid file \"%s\". reason: %s",
-						pool_config->pid_file_name, strerror(errno))));
+				(errmsg("EOF detected while reading pid file \"%s\"",
+						pool_config->pid_file_name),
+				 errdetail("%m")));
 	}
 	pfree(pid_file);
 	close(fd);
@@ -651,8 +678,9 @@ write_pid_file(void)
 	if (fd == -1)
 	{
 		ereport(FATAL,
-				(errmsg("could not open pid file as %s. reason: %s",
-						pool_config->pid_file_name, strerror(errno))));
+				(errmsg("could not open pid file \"%s\"",
+						pool_config->pid_file_name),
+				 errdetail("%m")));
 	}
 	snprintf(pidbuf, sizeof(pidbuf), "%d", (int) getpid());
 	if (write(fd, pidbuf, strlen(pidbuf) + 1) == -1)
@@ -660,23 +688,26 @@ write_pid_file(void)
 		close(fd);
 		pfree(pid_file);
 		ereport(FATAL,
-				(errmsg("could not write pid file as %s. reason: %s",
-						pool_config->pid_file_name, strerror(errno))));
+				(errmsg("could not write pid file \"%s\"",
+						pool_config->pid_file_name),
+				 errdetail("%m")));
 	}
 	if (fsync(fd) == -1)
 	{
 		close(fd);
 		pfree(pid_file);
 		ereport(FATAL,
-				(errmsg("could not fsync pid file as %s. reason: %s",
-						pool_config->pid_file_name, strerror(errno))));
+				(errmsg("could not fsync pid file \"%s\"",
+						pool_config->pid_file_name),
+				 errdetail("%m")));
 	}
 	if (close(fd) == -1)
 	{
 		pfree(pid_file);
 		ereport(FATAL,
-				(errmsg("could not close pid file as %s. reason: %s",
-						pool_config->pid_file_name, strerror(errno))));
+				(errmsg("could not close pid file \"%s\"",
+						pool_config->pid_file_name),
+				 errdetail("%m")));
 	}
 	/* register the call back to delete the pid file at system exit */
 	on_proc_exit(FileUnlink, (Datum) pid_file);
@@ -717,5 +748,5 @@ FileUnlink(int code, Datum path)
 	 */
 	ereport(LOG,
 			(errmsg("unlink failed for file at path \"%s\"", filePath),
-			 errdetail("\"%s\"", strerror(errno))));
+			 errdetail("%m")));
 }
