@@ -5,7 +5,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2015	PgPool Global Development Group
+ * Copyright (c) 2003-2021	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -25,6 +25,14 @@
 #include "pool.h"
 #include "utils/palloc.h"
 #include "utils/memutils.h"
+#include "utils/pool_signal.h"
+#include "utils/pool_ipc.h"
+#include "utils/ps_status.h"
+
+#include "context/pool_process_context.h"
+#include "context/pool_session_context.h"
+
+#include "pcp/pcp_worker.h"
 
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -132,6 +140,9 @@ pcp_main(int unix_fd, int inet_fd)
 
 	MemoryContextSwitchTo(TopMemoryContext);
 
+	/* Initialize per process context */
+	pool_init_process_context();
+
 	if (sigsetjmp(local_sigjmp_buf, 1) != 0)
 	{
 		error_context_stack = NULL;
@@ -190,7 +201,7 @@ pcp_do_accept(int unix_fd, int inet_fd)
 			return -1;
 		ereport(ERROR,
 				(errmsg("unable to accept new pcp connection"),
-				 errdetail("select system call failed with error : \"%s\"", strerror(errno))));
+				 errdetail("select system call failed with error : \"%m\"")));
 	}
 	if (FD_ISSET(unix_fd, &readmask))
 	{
@@ -214,7 +225,7 @@ pcp_do_accept(int unix_fd, int inet_fd)
 		if (errno != EAGAIN && errno != EWOULDBLOCK)
 			ereport(ERROR,
 					(errmsg("unable to accept new pcp connection"),
-					 errdetail("socket accept system call failed with error : \"%s\"", strerror(errno))));
+					 errdetail("socket accept system call failed with error : \"%m\"")));
 	}
 	if (pcp_got_sighup)
 	{
@@ -237,7 +248,7 @@ pcp_do_accept(int unix_fd, int inet_fd)
 			close(afd);
 			ereport(ERROR,
 					(errmsg("unable to accept new pcp connection"),
-					 errdetail("setsockopt system call failed with error : \"%s\"", strerror(errno))));
+					 errdetail("setsockopt system call failed with error : \"%m\"")));
 		}
 		if (setsockopt(afd, SOL_SOCKET, SO_KEEPALIVE,
 					   (char *) &on,
@@ -246,7 +257,7 @@ pcp_do_accept(int unix_fd, int inet_fd)
 			close(afd);
 			ereport(ERROR,
 					(errmsg("unable to accept new pcp connection"),
-					 errdetail("setsockopt system call failed with error : \"%s\"", strerror(errno))));
+					 errdetail("setsockopt system call failed with error : \"%m\"")));
 		}
 	}
 	return afd;
@@ -262,8 +273,8 @@ start_pcp_command_processor_process(int port)
 
 	if (pid == 0)				/* child */
 	{
-		/* Set the process type variable */
-		processType = PT_PCP_WORKER;
+		SetProcessGlobalVariables(PT_PCP_WORKER);
+
 		on_exit_reset();
 		/* Close the listen sockets sockets */
 		close(pcp_unix_fd);
@@ -278,7 +289,8 @@ start_pcp_command_processor_process(int port)
 	else if (pid == -1)
 	{
 		ereport(FATAL,
-				(errmsg("fork() failed. reason: %s", strerror(errno))));
+				(errmsg("fork() failed"),
+				 errdetail("%m")));
 	}
 	else						/* parent */
 	{
@@ -437,7 +449,8 @@ pcp_child_will_die(int code, Datum arg)
 
 	if (wpid == -1 && errno != ECHILD)
 		ereport(WARNING,
-				(errmsg("wait() on pcp worker children failed. reason:%s", strerror(errno))));
+				(errmsg("wait() on pcp worker children failed"),
+				 errdetail("%m")));
 
 	POOL_SETMASK(&UnBlockSig);
 }
@@ -454,7 +467,7 @@ pcp_mark_recovery_in_progress(void)
 	pool_sigset_t oldmask;
 
 	/*
-	 * only pcp worker is allowd to make this call
+	 * only pcp worker is allowed to make this call
 	 */
 	if (processType != PT_PCP_WORKER)
 		return false;
@@ -470,7 +483,7 @@ pcp_mark_recovery_in_progress(void)
 
 /*
  * unsets the shared memory flag to indicate pcp recovery command
- * is finsihed.
+ * is finished.
  */
 void
 pcp_mark_recovery_finished(void)
@@ -478,7 +491,7 @@ pcp_mark_recovery_finished(void)
 	pool_sigset_t oldmask;
 
 	/*
-	 * only pcp worker is allowd to make this call
+	 * only pcp worker is allowed to make this call
 	 */
 	if (processType != PT_PCP_WORKER)
 		return;
