@@ -4,7 +4,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2019	PgPool Global Development Group
+ * Copyright (c) 2003-2022	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -21,10 +21,8 @@
  *
  */
 
+
 #include "config.h"
-#include "pool.h"
-#include "utils/palloc.h"
-#include "utils/memutils.h"
 
 #include <arpa/inet.h>
 #include <signal.h>
@@ -40,15 +38,26 @@
 #include <fcntl.h>
 #endif
 
+#include "pool.h"
+#include "pool_config.h"
+
 #include "pcp/pcp_stream.h"
 #include "pcp/pcp.h"
+#include "pcp/pcp_worker.h"
+#include "pcp/recovery.h"
 #include "auth/md5.h"
-#include "pool_config.h"
+#include "auth/pool_auth.h"
 #include "context/pool_process_context.h"
+#include "context/pool_session_context.h"
 #include "utils/pool_process_reporting.h"
-#include "watchdog/wd_json_data.h"
-#include "watchdog/wd_ipc_commands.h"
+#include "utils/palloc.h"
+#include "utils/memutils.h"
+#include "utils/ps_status.h"
 #include "utils/elog.h"
+#include "watchdog/wd_json_data.h"
+#include "watchdog/wd_internal_commands.h"
+#include "main/pool_internal_comms.h"
+
 
 #define MAX_FILE_LINE_LEN    512
 
@@ -67,20 +76,22 @@ static void send_md5salt(PCP_CONNECTION * frontend, char *salt);
 
 static void pcp_process_command(char tos, char *buf, int buf_len);
 
-static int	pool_detach_node(int node_id, bool gracefully);
+static int	pool_detach_node(int node_id, bool gracefully, bool switchover);
 static int	pool_promote_node(int node_id, bool gracefully);
 static void inform_process_count(PCP_CONNECTION * frontend);
 static void inform_process_info(PCP_CONNECTION * frontend, char *buf);
 static void inform_watchdog_info(PCP_CONNECTION * frontend, char *buf);
 static void inform_node_info(PCP_CONNECTION * frontend, char *buf);
 static void inform_node_count(PCP_CONNECTION * frontend);
+static void process_reload_config(PCP_CONNECTION * frontend,char scope);
+static void inform_health_check_stats(PCP_CONNECTION *frontend, char *buf);
 static void process_detach_node(PCP_CONNECTION * frontend, char *buf, char tos);
 static void process_attach_node(PCP_CONNECTION * frontend, char *buf);
 static void process_recovery_request(PCP_CONNECTION * frontend, char *buf);
 static void process_status_request(PCP_CONNECTION * frontend);
 static void process_promote_node(PCP_CONNECTION * frontend, char *buf, char tos);
-static void process_shutown_request(PCP_CONNECTION * frontend, char mode);
-static void process_set_configration_parameter(PCP_CONNECTION * frontend, char *buf, int len);
+static void process_shutdown_request(PCP_CONNECTION * frontend, char mode, char tos);
+static void process_set_configuration_parameter(PCP_CONNECTION * frontend, char *buf, int len);
 
 static void pcp_worker_will_go_down(int code, Datum arg);
 
@@ -217,9 +228,10 @@ pcp_worker_main(int port)
 static void
 pcp_process_command(char tos, char *buf, int buf_len)
 {
-
+	/* The request is recovery or pcp shutdown request? */
 	if (tos == 'O' || tos == 'T')
 	{
+		/* Prevent those pcp requests while processing failover/failback request */
 		if (Req_info->switching)
 		{
 			if (Req_info->request_queue_tail != Req_info->request_queue_head)
@@ -251,8 +263,8 @@ pcp_process_command(char tos, char *buf, int buf_len)
 	switch (tos)
 	{
 		case 'A':				/* set configuration parameter */
-			set_ps_display("PCP: processing set configration parameter request", false);
-			process_set_configration_parameter(pcp_frontend, buf, buf_len);
+			set_ps_display("PCP: processing set configuration parameter request", false);
+			process_set_configuration_parameter(pcp_frontend, buf, buf_len);
 			break;
 
 		case 'L':				/* node count */
@@ -260,6 +272,11 @@ pcp_process_command(char tos, char *buf, int buf_len)
 			inform_node_count(pcp_frontend);
 			break;
 
+		case 'H':				/* health check stats */
+			set_ps_display("PCP: processing health check stats request", false);
+			inform_health_check_stats(pcp_frontend, buf);
+			break;
+
 		case 'I':				/* node info */
 			set_ps_display("PCP: processing node info request", false);
 			inform_node_info(pcp_frontend, buf);
@@ -292,8 +309,9 @@ pcp_process_command(char tos, char *buf, int buf_len)
 			break;
 
 		case 'T':
+		case 't':
 			set_ps_display("PCP: processing shutdown request", false);
-			process_shutown_request(pcp_frontend, buf[0]);
+			process_shutdown_request(pcp_frontend, buf[0], tos);
 			break;
 
 		case 'O':				/* recovery request */
@@ -306,6 +324,11 @@ pcp_process_command(char tos, char *buf, int buf_len)
 			process_status_request(pcp_frontend);
 			break;
 
+		case 'Z':				/*reload config file */
+			set_ps_display("PCP: processing reload config request", false);
+			process_reload_config(pcp_frontend, buf[0]);
+			break;
+
 		case 'J':				/* promote node */
 		case 'j':				/* promote node gracefully */
 			set_ps_display("PCP: processing promote node request", false);
@@ -382,14 +405,14 @@ unset_nonblock(int fd)
 	{
 		ereport(FATAL,
 				(errmsg("unable to connect"),
-				 errdetail("fcntl system call failed with error : \"%s\"", strerror(errno))));
+				 errdetail("fcntl system call failed with error : \"%m\"")));
 
 	}
 	if (fcntl(fd, F_SETFL, var & ~O_NONBLOCK) == -1)
 	{
 		ereport(FATAL,
 				(errmsg("unable to connect"),
-				 errdetail("fcntl system call failed with error : \"%s\"", strerror(errno))));
+				 errdetail("fcntl system call failed with error : \"%m\"")));
 	}
 }
 
@@ -427,7 +450,7 @@ user_authenticate(char *buf, char *passwd_file, char *salt, int salt_len)
 	{
 		ereport(FATAL,
 				(errmsg("failed to authenticate PCP user"),
-				 errdetail("could not open %s. reason: %s", passwd_file, strerror(errno))));
+				 errdetail("could not open %s. reason: %m", passwd_file)));
 		return 0;
 	}
 
@@ -501,11 +524,16 @@ user_authenticate(char *buf, char *passwd_file, char *salt, int salt_len)
 
 /* Detach a node */
 static int
-pool_detach_node(int node_id, bool gracefully)
+pool_detach_node(int node_id, bool gracefully, bool switchover)
 {
+	int		flag = 0;
+
+	if (switchover)
+		flag = REQ_DETAIL_PROMOTE;
+
 	if (!gracefully)
 	{
-		degenerate_backend_set_ex(&node_id, 1, REQ_DETAIL_SWITCHOVER | REQ_DETAIL_CONFIRMED, true, false);
+		degenerate_backend_set_ex(&node_id, 1, flag | REQ_DETAIL_SWITCHOVER | REQ_DETAIL_CONFIRMED, true, false);
 		return 0;
 	}
 
@@ -517,7 +545,7 @@ pool_detach_node(int node_id, bool gracefully)
 	/*
 	 * Wait until all frontends exit
 	 */
-	*InRecovery = RECOVERY_DETACH;	/* This wiil ensure that new incoming
+	*InRecovery = RECOVERY_DETACH;	/* This will ensure that new incoming
 									 * connection requests are blocked */
 
 	if (wait_connection_closed())
@@ -532,7 +560,7 @@ pool_detach_node(int node_id, bool gracefully)
 	/*
 	 * Now all frontends have gone. Let's do failover.
 	 */
-	degenerate_backend_set_ex(&node_id, 1, REQ_DETAIL_SWITCHOVER | REQ_DETAIL_CONFIRMED, false, false);
+	degenerate_backend_set_ex(&node_id, 1, flag | REQ_DETAIL_SWITCHOVER | REQ_DETAIL_CONFIRMED, false, false);
 
 	/*
 	 * Wait for failover completed.
@@ -568,7 +596,7 @@ pool_promote_node(int node_id, bool gracefully)
 	/*
 	 * Wait until all frontends exit
 	 */
-	*InRecovery = RECOVERY_PROMOTE; /* This wiil ensure that new incoming
+	*InRecovery = RECOVERY_PROMOTE; /* This will ensure that new incoming
 									 * connection requests are blocked */
 
 	if (wait_connection_closed())
@@ -618,14 +646,13 @@ inform_process_count(PCP_CONNECTION * frontend)
 
 	process_list = pool_get_process_list(&process_count);
 
-	mesg = (char *) palloc(7 * process_count);	/* PID is at most 6 characters
-												 * long */
+	mesg = (char *) palloc(8 * process_count);	/* PID is at most 7 characters long */
 
 	snprintf(process_count_str, sizeof(process_count_str), "%d", process_count);
 
 	for (i = 0; i < process_count; i++)
 	{
-		char		process_id[7];
+		char		process_id[8];
 
 		snprintf(process_id, sizeof(process_id), "%d", process_list[i]);
 		snprintf(mesg + total_port_len, strlen(process_id) + 1, "%s", process_id);
@@ -651,6 +678,9 @@ inform_process_count(PCP_CONNECTION * frontend)
 			 errdetail("%d process(es) found", process_count)));
 }
 
+/*
+ * pcp_process_info
+ */
 static void
 inform_process_info(PCP_CONNECTION * frontend, char *buf)
 {
@@ -658,6 +688,9 @@ inform_process_info(PCP_CONNECTION * frontend, char *buf)
 	int			wsize;
 	int			num_proc = pool_config->num_init_children;
 	int			i;
+	int			*offsets;
+	int			n;
+	POOL_REPORT_POOLS *pools;
 
 	proc_id = atoi(buf);
 
@@ -676,7 +709,7 @@ inform_process_info(PCP_CONNECTION * frontend, char *buf)
 		/* Finally, indicate that all data is sent */
 		char		fin_code[] = "CommandComplete";
 
-		POOL_REPORT_POOLS *pools = get_pools(&num_proc);
+		pools = get_pools(&num_proc);
 
 		if (proc_id == 0)
 		{
@@ -696,60 +729,41 @@ inform_process_info(PCP_CONNECTION * frontend, char *buf)
 		pcp_write(frontend, con_info_size, strlen(con_info_size) + 1);
 		do_pcp_flush(frontend);
 
+		offsets = pool_report_pools_offsets(&n);
+
 		/* Second, send process information for all connection_info */
 		for (i = 0; i < num_proc; i++)
 		{
+			int			j;
 			char		code[] = "ProcessInfo";
-			char		proc_pid[16];
-			char		proc_start_time[20];
-			char		proc_create_time[20];
-			char		majorversion[5];
-			char		minorversion[5];
-			char		pool_counter[16];
-			char		backend_id[16];
-			char		backend_pid[16];
-			char		connected[2];
+			int			pool_pid;
 
-			if (proc_id != 0 && proc_id != pools[i].pool_pid)
+			pool_pid = atoi(pools[i].pool_pid);
+
+			if (proc_id != 0 && pool_pid != proc_id)
 				continue;
 
-			snprintf(proc_pid, sizeof(proc_pid), "%d", pools[i].pool_pid);
-			snprintf(proc_start_time, sizeof(proc_start_time), "%ld", pools[i].start_time);
-			snprintf(proc_create_time, sizeof(proc_create_time), "%ld", pools[i].create_time);
-			snprintf(majorversion, sizeof(majorversion), "%d", pools[i].pool_majorversion);
-			snprintf(minorversion, sizeof(minorversion), "%d", pools[i].pool_minorversion);
-			snprintf(pool_counter, sizeof(pool_counter), "%d", pools[i].pool_counter);
-			snprintf(backend_id, sizeof(backend_pid), "%d", pools[i].backend_id);
-			snprintf(backend_pid, sizeof(backend_pid), "%d", pools[i].pool_backendpid);
-			snprintf(connected, sizeof(connected), "%d", pools[i].pool_connected);
-
 			pcp_write(frontend, "p", 1);
-			wsize = htonl(sizeof(code) +
-						  strlen(proc_pid) + 1 +
-						  strlen(pools[i].database) + 1 +
-						  strlen(pools[i].username) + 1 +
-						  strlen(proc_start_time) + 1 +
-						  strlen(proc_create_time) + 1 +
-						  strlen(majorversion) + 1 +
-						  strlen(minorversion) + 1 +
-						  strlen(pool_counter) + 1 +
-						  strlen(backend_id) + 1 +
-						  strlen(backend_pid) + 1 +
-						  strlen(connected) + 1 +
-						  sizeof(int));
+
+			wsize = 0;
+			for (j = 0; j < n; j++)
+			{
+				wsize += strlen((char *)&pools[i] + offsets[j]) + 1;
+			}
+			wsize += sizeof(code) + sizeof(int);
+			wsize = htonl(wsize);
+
+			/* send packet length to frontend */
 			pcp_write(frontend, &wsize, sizeof(int));
+			/* send "this is a record" to frontend */
 			pcp_write(frontend, code, sizeof(code));
-			pcp_write(frontend, proc_pid, strlen(proc_pid) + 1);
-			pcp_write(frontend, pools[i].database, strlen(pools[i].database) + 1);
-			pcp_write(frontend, pools[i].username, strlen(pools[i].username) + 1);
-			pcp_write(frontend, proc_start_time, strlen(proc_start_time) + 1);
-			pcp_write(frontend, proc_create_time, strlen(proc_create_time) + 1);
-			pcp_write(frontend, majorversion, strlen(majorversion) + 1);
-			pcp_write(frontend, minorversion, strlen(minorversion) + 1);
-			pcp_write(frontend, pool_counter, strlen(pool_counter) + 1);
-			pcp_write(frontend, backend_id, strlen(backend_id) + 1);
-			pcp_write(frontend, backend_pid, strlen(backend_pid) + 1);
-			pcp_write(frontend, connected, strlen(connected) + 1);
+
+			/* send each process info data to frontend */
+			for (j = 0; j < n; j++)
+			{
+				pcp_write(frontend, (char *)&pools[i] + offsets[j], strlen((char *)&pools[i] + offsets[j]) + 1);
+			}
+
 			do_pcp_flush(frontend);
 		}
 
@@ -779,11 +793,11 @@ inform_watchdog_info(PCP_CONNECTION * frontend, char *buf)
 	if (!pool_config->use_watchdog)
 		ereport(ERROR,
 				(errmsg("PCP: informing watchdog info failed"),
-				 errdetail("watcdhog is not enabled")));
+				 errdetail("watchdog is not enabled")));
 
 	wd_index = atoi(buf);
 
-	json_data = wd_get_watchdog_nodes(wd_index);
+	json_data = wd_internal_get_watchdog_nodes_json(wd_index);
 	if (json_data == NULL)
 		ereport(ERROR,
 				(errmsg("PCP: informing watchdog info failed"),
@@ -794,8 +808,8 @@ inform_watchdog_info(PCP_CONNECTION * frontend, char *buf)
 			 errdetail("retrieved node information from IPC socket")));
 
 	/*
-	 * This is the voilation of PCP protocol but I think in future we should
-	 * shift to more adaptable protocol for data transmition.
+	 * This is the violation of PCP protocol but I think in future we should
+	 * shift to more adaptable protocol for data transmission.
 	 */
 	json_data_len = strlen(json_data);
 	wsize = htonl(sizeof(code) +
@@ -815,79 +829,220 @@ inform_watchdog_info(PCP_CONNECTION * frontend, char *buf)
 static void
 inform_node_info(PCP_CONNECTION * frontend, char *buf)
 {
+	POOL_REPORT_NODES *nodes;
+	int			nrows;
 	int			node_id;
 	int			wsize;
-	char		port_str[6];
-	char		status[2];
-	char		weight_str[20];
-	char		role_str[10];
-	char		standby_delay_str[20];
-	char		status_changed_time_str[20];
-	char		code[] = "CommandComplete";
-	BackendInfo *bi = NULL;
-	SERVER_ROLE role;
+	int			i;
+
 
 	node_id = atoi(buf);
 
-	bi = pool_get_node_info(node_id);
-
-	if (bi == NULL)
+	if ((node_id != -1) && (pool_get_node_info(node_id) == NULL))
+	{
 		ereport(ERROR,
 				(errmsg("informing node info failed"),
-				 errdetail("invalid node ID")));
-
-	ereport(DEBUG2,
-			(errmsg("PCP: informing node info"),
-			 errdetail("retrieved node information from shared memory")));
-
-	snprintf(port_str, sizeof(port_str), "%d", bi->backend_port);
-	snprintf(status, sizeof(status), "%d", bi->backend_status);
-	snprintf(weight_str, sizeof(weight_str), "%f", bi->backend_weight);
-
-	if (STREAM)
-	{
-		if (Req_info->primary_node_id == node_id)
-			role = ROLE_PRIMARY;
-		else
-			role = ROLE_STANDBY;
+				 errdetail("invalid node ID : %s", buf)));
 	}
 	else
 	{
-		if (Req_info->master_node_id == node_id)
-			role = ROLE_MASTER;
+		/* First, send array size of node_info */
+		char		arr_code[] = "ArraySize";
+		char		node_info_size[16];
+
+		/* Finally, indicate that all data is sent */
+		char		fin_code[] = "CommandComplete";
+
+		nodes = get_nodes(&nrows, node_id);
+
+		if (node_id == -1)
+		{
+			snprintf(node_info_size, sizeof(node_info_size), "%d", NUM_BACKENDS);
+		}
 		else
-			role = ROLE_SLAVE;
+		{
+			snprintf(node_info_size, sizeof(node_info_size), "%d", 1);
+		}
+
+		pcp_write(frontend, "i", 1);
+		wsize = htonl(sizeof(arr_code) +
+					  strlen(node_info_size) + 1 +
+					  sizeof(int));
+		pcp_write(frontend, &wsize, sizeof(int));
+		pcp_write(frontend, arr_code, sizeof(arr_code));
+		pcp_write(frontend, node_info_size, strlen(node_info_size) + 1);
+		do_pcp_flush(frontend);
+
+		/* Second, send process information for all connection_info */
+		for (i = 0; i < NUM_BACKENDS ; i++)
+		{
+			char		port_str[6];
+			char		status[2];
+			char		quarantine[2];
+			char		weight_str[20];
+			char		role_str[10];
+			char		standby_delay_str[20];
+			char		status_changed_time_str[20];
+			char		code[] = "NodeInfo";
+			BackendInfo *bi = NULL;
+			SERVER_ROLE role;
+
+			if (node_id != -1 && node_id != i)
+				continue;
+
+			bi = pool_get_node_info(i);
+
+			if (bi == NULL)
+				ereport(ERROR,
+						(errmsg("informing node info failed"),
+						errdetail("invalid node ID")));
+
+			snprintf(port_str, sizeof(port_str), "%d", bi->backend_port);
+			snprintf(status, sizeof(status), "%d", bi->backend_status);
+			snprintf(quarantine, sizeof(quarantine), "%d", bi->quarantine);
+			snprintf(weight_str, sizeof(weight_str), "%f", bi->backend_weight);
+
+			if (STREAM)
+			{
+				if (Req_info->primary_node_id == i)
+					role = ROLE_PRIMARY;
+				else
+					role = ROLE_STANDBY;
+			}
+			else
+			{
+				if (Req_info->main_node_id == i)
+					role = ROLE_MAIN;
+				else
+					role = ROLE_REPLICA;
+			}
+			snprintf(role_str, sizeof(role_str), "%d", role);
+
+			snprintf(standby_delay_str, sizeof(standby_delay_str), UINT64_FORMAT, bi->standby_delay);
+
+			snprintf(status_changed_time_str, sizeof(status_changed_time_str), UINT64_FORMAT, bi->status_changed_time);
+
+			pcp_write(frontend, "i", 1);
+			wsize = htonl(sizeof(code) +
+						  strlen(bi->backend_hostname) + 1 +
+						  strlen(port_str) + 1 +
+						  strlen(status) + 1 +
+						  strlen(quarantine) + 1 +
+						  strlen(nodes[i].pg_status) + 1 +
+						  strlen(weight_str) + 1 +
+						  strlen(role_str) + 1 +
+						  strlen(nodes[i].pg_role) + 1 +
+						  strlen(standby_delay_str) + 1 +
+						  strlen(bi->replication_state) + 1 +
+						  strlen(bi->replication_sync_state) + 1 +
+						  strlen(status_changed_time_str) + 1 +
+						  sizeof(int));
+			pcp_write(frontend, &wsize, sizeof(int));
+			pcp_write(frontend, code, sizeof(code));
+			pcp_write(frontend, bi->backend_hostname, strlen(bi->backend_hostname) + 1);
+			pcp_write(frontend, port_str, strlen(port_str) + 1);
+			pcp_write(frontend, status, strlen(status) + 1);
+			pcp_write(frontend, quarantine, strlen(quarantine) + 1);
+			pcp_write(frontend, nodes[i].pg_status, strlen(nodes[i].pg_status) + 1);
+			pcp_write(frontend, weight_str, strlen(weight_str) + 1);
+			pcp_write(frontend, role_str, strlen(role_str) + 1);
+			pcp_write(frontend, nodes[i].pg_role, strlen(nodes[i].pg_role) + 1);
+			pcp_write(frontend, standby_delay_str, strlen(standby_delay_str) + 1);
+			pcp_write(frontend, bi->replication_state, strlen(bi->replication_state) + 1);
+			pcp_write(frontend, bi->replication_sync_state, strlen(bi->replication_sync_state) + 1);
+			pcp_write(frontend, status_changed_time_str, strlen(status_changed_time_str) + 1);
+			do_pcp_flush(frontend);
+		}
+
+		pcp_write(frontend, "i", 1);
+		wsize = htonl(sizeof(fin_code) +
+					  sizeof(int));
+		pcp_write(frontend, &wsize, sizeof(int));
+		pcp_write(frontend, fin_code, sizeof(fin_code));
+		do_pcp_flush(frontend);
+		ereport(DEBUG1,
+				(errmsg("PCP informing node info"),
+				 errdetail("retrieved node information from shared memory")));
+
+		pfree(nodes);
 	}
-	snprintf(role_str, sizeof(role_str), "%d", role);
+}
 
-	snprintf(standby_delay_str, sizeof(standby_delay_str), UINT64_FORMAT, bi->standby_delay);
+/*
+ * Send out health check stats data to pcp client.  node id is provided as a
+ * string in buf parameter.
+ *
+ * The protocol starts with 'h', followed by 4-byte packet length integer in
+ * network byte order including self.  Each data is represented as a null
+ * terminated string. The order of each data is defined in
+ * POOL_HEALTH_CHECK_STATS struct.
+ */
+static void
+inform_health_check_stats(PCP_CONNECTION *frontend, char *buf)
+{
+	POOL_HEALTH_CHECK_STATS *stats;
+	POOL_HEALTH_CHECK_STATS *s;
+	int		*offsets;
+	int		n;
+	int		nrows;
+	int		i;
+	int		node_id;
+	bool	node_id_ok = false;
+	int		wsize;
+	char	code[] = "CommandComplete";
 
-	snprintf(status_changed_time_str, sizeof(status_changed_time_str), UINT64_FORMAT, bi->status_changed_time);
+	node_id = atoi(buf);
 
-	pcp_write(frontend, "i", 1);
-	wsize = htonl(sizeof(code) +
-				  strlen(bi->backend_hostname) + 1 +
-				  strlen(port_str) + 1 +
-				  strlen(status) + 1 +
-				  strlen(weight_str) + 1 +
-				  strlen(role_str) + 1 +
-				  strlen(standby_delay_str) + 1 +
-				  strlen(bi->replication_state) + 1 +
-				  strlen(bi->replication_sync_state) + 1 +
-				  strlen(status_changed_time_str) + 1 +
-				  sizeof(int));
+	if (node_id < 0 || node_id > NUM_BACKENDS)
+	{
+		ereport(ERROR,
+				(errmsg("informing health check stats info failed"),
+				 errdetail("invalid node ID %d", node_id)));
+	}
+	
+	stats = get_health_check_stats(&nrows);
+
+	for (i = 0; i < nrows; i++)
+	{
+		if (atoi(stats[i].node_id) == node_id)
+		{
+			node_id_ok = true;
+			s = &stats[i];
+			break;
+		}
+	}
+
+	if (!node_id_ok)
+	{
+		ereport(ERROR,
+				(errmsg("informing health check stats info failed"),
+				 errdetail("stats data for node ID %d does not exist", node_id)));
+	}
+
+	pcp_write(frontend, "h", 1);	/* indicate that this is a reply to health check stats request */
+
+	wsize = sizeof(code) + sizeof(int);
+
+	/* Calculate total packet length */
+	offsets = pool_health_check_stats_offsets(&n);
+
+	for (i = 0; i < n; i++)
+	{
+		wsize += strlen((char *)s + offsets[i]) + 1;
+	}
+	wsize = htonl(wsize);	/* convert to network byte order */
+
+	/* send packet length to frontend */
 	pcp_write(frontend, &wsize, sizeof(int));
+	/* send "Command Complete" to frontend */
 	pcp_write(frontend, code, sizeof(code));
-	pcp_write(frontend, bi->backend_hostname, strlen(bi->backend_hostname) + 1);
-	pcp_write(frontend, port_str, strlen(port_str) + 1);
-	pcp_write(frontend, status, strlen(status) + 1);
-	pcp_write(frontend, weight_str, strlen(weight_str) + 1);
-	pcp_write(frontend, role_str, strlen(role_str) + 1);
-	pcp_write(frontend, standby_delay_str, strlen(standby_delay_str) + 1);
-	pcp_write(frontend, bi->replication_state, strlen(bi->replication_state) + 1);
-	pcp_write(frontend, bi->replication_sync_state, strlen(bi->replication_sync_state) + 1);
-	pcp_write(frontend, status_changed_time_str, strlen(status_changed_time_str) + 1);
 
+	/* send each health check stats data to frontend */
+	for (i = 0; i < n; i++)
+	{
+		pcp_write(frontend, (char *)s + offsets[i], strlen((char *)s + offsets[i]) + 1);
+	}
+	pfree(stats);
 	do_pcp_flush(frontend);
 }
 
@@ -914,6 +1069,36 @@ inform_node_count(PCP_CONNECTION * frontend)
 			(errmsg("PCP: informing node count"),
 			 errdetail("%d node(s) found", node_count)));
 }
+static void
+process_reload_config(PCP_CONNECTION * frontend, char scope)
+{
+	char            code[] = "CommandComplete";
+	int wsize;
+
+	if (scope == 'c' && pool_config->use_watchdog)
+	{
+		ereport(LOG,
+				(errmsg("PCP: sending command to watchdog to reload config cluster")));
+
+		if (wd_execute_cluster_command(WD_COMMAND_RELOAD_CONFIG_CLUSTER, NULL) != COMMAND_OK)
+			ereport(ERROR,
+					(errmsg("PCP: error while processing reload config request for cluster"),
+					 errdetail("failed to propagate reload config command through watchdog")));
+	}
+
+	if(pool_signal_parent(SIGHUP) == -1)
+	{
+	   ereport(ERROR,
+			   (errmsg("process reload config request failed"),
+				errdetail("failed to signal pgpool parent process")));
+	}
+
+	pcp_write(frontend, "z", 1);
+	wsize = htonl(sizeof(code) + sizeof(int));
+	pcp_write(frontend, &wsize, sizeof(int));
+	pcp_write(frontend, code, sizeof(code));
+	do_pcp_flush(frontend);
+}
 
 static void
 process_detach_node(PCP_CONNECTION * frontend, char *buf, char tos)
@@ -933,7 +1118,7 @@ process_detach_node(PCP_CONNECTION * frontend, char *buf, char tos)
 			(errmsg("PCP: processing detach node"),
 			 errdetail("detaching Node ID %d", node_id)));
 
-	pool_detach_node(node_id, gracefully);
+	pool_detach_node(node_id, gracefully, false);
 
 	pcp_write(frontend, "d", 1);
 	wsize = htonl(sizeof(code) + sizeof(int));
@@ -977,13 +1162,11 @@ process_recovery_request(PCP_CONNECTION * frontend, char *buf)
 				 errdetail("node id %d is not valid", node_id)));
 
 	if ((!REPLICATION &&
-		 !(MASTER_SLAVE &&
-		   pool_config->master_slave_sub_mode == STREAM_MODE)) ||
-		(MASTER_SLAVE &&
-		 pool_config->master_slave_sub_mode == STREAM_MODE &&
+		 !(STREAM)) ||
+		(STREAM &&
 		 node_id == PRIMARY_NODE_ID))
 	{
-		if (MASTER_SLAVE && pool_config->master_slave_sub_mode == STREAM_MODE)
+		if (STREAM)
 			ereport(ERROR,
 					(errmsg("process recovery request failed"),
 					 errdetail("primary server cannot be recovered by online recovery.")));
@@ -1078,6 +1261,11 @@ process_status_request(PCP_CONNECTION * frontend)
 			 errdetail("retrieved status information")));
 }
 
+/*
+ * Process promote node request.  This function is tricky. If promote option
+ * is sent from client, calls pool_detach_node() so that failover script
+ * actually promote the specified node and detach current primary.
+ */
 static void
 process_promote_node(PCP_CONNECTION * frontend, char *buf, char tos)
 {
@@ -1085,22 +1273,34 @@ process_promote_node(PCP_CONNECTION * frontend, char *buf, char tos)
 	int			wsize;
 	char		code[] = "CommandComplete";
 	bool		gracefully;
+	char		node_id_buf[64];
+	char		*p;
+	char		promote_option;
 
 	if (tos == 'J')
 		gracefully = false;
 	else
 		gracefully = true;
 
-	node_id = atoi(buf);
+	p = node_id_buf;
+	while (*buf && *buf != ' ')
+		*p++ = *buf++;
+	*p = '\0';
+	buf += 2;
+	promote_option = *buf;
+
+	node_id = atoi(node_id_buf);
+
 	if ((node_id < 0) || (node_id >= pool_config->backend_desc->num_backends))
 		ereport(ERROR,
-				(errmsg("could not process recovery request"),
+				(errmsg("could not process promote request"),
 				 errdetail("node id %d is not valid", node_id)));
-	/* promoting node is reserved to Streaming Replication */
-	if (!MASTER_SLAVE || pool_config->master_slave_sub_mode != STREAM_MODE)
+
+	/* Promoting node is only possible in Streaming Replication mode */
+	if (!STREAM)
 	{
 		ereport(FATAL,
-				(errmsg("invalid pgpool mode for process recovery request"),
+				(errmsg("invalid pgpool mode for process promote request"),
 				 errdetail("not in streaming replication mode, can't promote node id %d", node_id)));
 
 	}
@@ -1108,14 +1308,27 @@ process_promote_node(PCP_CONNECTION * frontend, char *buf, char tos)
 	if (node_id == REAL_PRIMARY_NODE_ID)
 	{
 		ereport(FATAL,
-				(errmsg("invalid pgpool mode for process recovery request"),
+				(errmsg("invalid promote request"),
 				 errdetail("specified node is already primary node, can't promote node id %d", node_id)));
 
 	}
-	ereport(DEBUG1,
+
+	ereport(LOG, (errmsg("pcp_promote_node: promote option: %c", promote_option)));
+
+	if (promote_option == 's')
+	{
+		ereport(DEBUG1,
 			(errmsg("PCP: processing promote node"),
-			 errdetail("promoting Node ID %d", node_id)));
-	pool_promote_node(node_id, gracefully);
+			 errdetail("promoting Node ID %d and shutdown primary node %d", node_id, REAL_PRIMARY_NODE_ID)));
+		pool_detach_node(node_id, gracefully, true);
+	}
+	else
+	{
+		ereport(DEBUG1,
+				(errmsg("PCP: processing promote node"),
+				 errdetail("promoting Node ID %d", node_id)));
+		pool_promote_node(node_id, gracefully);
+	}
 
 	pcp_write(frontend, "d", 1);
 	wsize = htonl(sizeof(code) + sizeof(int));
@@ -1177,52 +1390,55 @@ send_md5salt(PCP_CONNECTION * frontend, char *salt)
 }
 
 static void
-process_shutown_request(PCP_CONNECTION * frontend, char mode)
+process_shutdown_request(PCP_CONNECTION * frontend, char mode, char tos)
 {
 	char		code[] = "CommandComplete";
-	pid_t		ppid = getppid();
-	int			sig,
-				len;
+	int			len;
 
-	if (mode == 's')
-	{
-		ereport(DEBUG1,
-				(errmsg("PCP: processing shutdown request"),
-				 errdetail("sending SIGTERM to the parent process with PID:%d", ppid)));
-		sig = SIGTERM;
-	}
-	else if (mode == 'f')
-	{
-		ereport(DEBUG1,
-				(errmsg("PCP: processing shutdown request"),
-				 errdetail("sending SIGINT to the parent process with PID:%d", ppid)));
-		sig = SIGINT;
-	}
-	else if (mode == 'i')
-	{
-		ereport(DEBUG1,
-				(errmsg("PCP: processing shutdown request"),
-				 errdetail("sending SIGQUIT to the parent process with PID:%d", ppid)));
-		sig = SIGQUIT;
-	}
-	else
+	ereport(DEBUG1,
+			(errmsg("PCP: processing shutdown request"),
+			 errdetail("shutdown mode \"%c\"", mode)));
+
+	/* quickly bail out if invalid mode is specified
+	 * because we do not want to propagate the command
+	 * with invalid mode over the watchdog network */
+	if (mode != 's' && mode != 'i' && mode != 'f' )
 	{
 		ereport(ERROR,
 				(errmsg("PCP: error while processing shutdown request"),
 				 errdetail("invalid shutdown mode \"%c\"", mode)));
 	}
 
+	if (tos == 't' && pool_config->use_watchdog)
+	{
+		WDExecCommandArg wdExecCommandArg;
+		List *args_list = NULL;
+
+		strncpy(wdExecCommandArg.arg_name, "mode", sizeof(wdExecCommandArg.arg_name) - 1);
+		snprintf(wdExecCommandArg.arg_value, sizeof(wdExecCommandArg.arg_value) - 1, "%c",mode);
+		args_list = lappend(args_list,&wdExecCommandArg);
+
+		ereport(LOG,
+				(errmsg("PCP: sending command to watchdog to shutdown cluster")));
+
+		if (wd_execute_cluster_command(WD_COMMAND_SHUTDOWN_CLUSTER, args_list) != COMMAND_OK)
+			ereport(ERROR,
+					(errmsg("PCP: error while processing shutdown cluster request"),
+					 errdetail("failed to propagate shutdown command through watchdog")));
+
+		list_free(args_list);
+	}
+
 	pcp_write(frontend, "t", 1);
 	len = htonl(sizeof(code) + sizeof(int));
 	pcp_write(frontend, &len, sizeof(int));
 	pcp_write(frontend, code, sizeof(code));
 	do_pcp_flush(frontend);
-
-	pool_signal_parent(sig);
+	terminate_pgpool(mode, true);
 }
 
 static void
-process_set_configration_parameter(PCP_CONNECTION * frontend, char *buf, int len)
+process_set_configuration_parameter(PCP_CONNECTION * frontend, char *buf, int len)
 {
 	char	   *param_name;
 	char	   *param_value;
@@ -1293,7 +1509,7 @@ do_pcp_flush(PCP_CONNECTION * frontend)
 	if (pcp_flush(frontend) < 0)
 		ereport(FATAL,
 				(errmsg("failed to flush data to client"),
-				 errdetail("pcp_flush failed with error : \"%s\"", strerror(errno))));
+				 errdetail("pcp_flush failed with error : \"%m\"")));
 }
 
 /*
@@ -1305,7 +1521,7 @@ do_pcp_read(PCP_CONNECTION * pc, void *buf, int len)
 	if (pcp_read(pc, buf, len))
 		ereport(FATAL,
 				(errmsg("unable to read from client"),
-				 errdetail("pcp_read failed with error : \"%s\"", strerror(errno))));
+				 errdetail("pcp_read failed with error : \"%m\"")));
 }
 
 int
