@@ -5,7 +5,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2019	PgPool Global Development Group
+ * Copyright (c) 2003-2021	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -29,6 +29,7 @@
 
 #include "pool.h"
 #include "protocol/pool_proto_modules.h"
+#include "protocol/pool_process_query.h"
 #include "parser/pg_config_manual.h"
 #include "parser/pool_string.h"
 #include "pool_config.h"
@@ -41,9 +42,9 @@
 
 static int	extract_ntuples(char *message);
 static POOL_STATUS handle_mismatch_tuples(POOL_CONNECTION * frontend, POOL_CONNECTION_POOL * backend, char *packet, int packetlen, bool command_complete);
-static int	foward_command_complete(POOL_CONNECTION * frontend, char *packet, int packetlen);
-static int	foward_empty_query(POOL_CONNECTION * frontend, char *packet, int packetlen);
-static int	foward_packet_to_frontend(POOL_CONNECTION * frontend, char kind, char *packet, int packetlen);
+static int	forward_command_complete(POOL_CONNECTION * frontend, char *packet, int packetlen);
+static int	forward_empty_query(POOL_CONNECTION * frontend, char *packet, int packetlen);
+static int	forward_packet_to_frontend(POOL_CONNECTION * frontend, char kind, char *packet, int packetlen);
 
 POOL_STATUS
 CommandComplete(POOL_CONNECTION * frontend, POOL_CONNECTION_POOL * backend, bool command_complete)
@@ -63,7 +64,7 @@ CommandComplete(POOL_CONNECTION * frontend, POOL_CONNECTION_POOL * backend, bool
 	session_context = pool_get_session_context(false);
 
 	/*
-	 * Handle misc process which is neccessary when query context exists.
+	 * Handle misc process which is necessary when query context exists.
 	 */
 	if (session_context->query_context != NULL && (!SL_MODE || (SL_MODE && !pool_is_doing_extended_query_message())))
 		handle_query_context(backend);
@@ -120,11 +121,11 @@ CommandComplete(POOL_CONNECTION * frontend, POOL_CONNECTION_POOL * backend, bool
 	}
 
 	/*
-	 * Otherwise just read from master node.
+	 * Otherwise just read from main node.
 	 */
 	else
 	{
-		con = MASTER(backend);
+		con = MAIN(backend);
 
 		if (pool_read(con, &len, sizeof(len)) < 0)
 			return POOL_END;
@@ -157,9 +158,9 @@ CommandComplete(POOL_CONNECTION * frontend, POOL_CONNECTION_POOL * backend, bool
 		}
 
 		if (command_complete)
-			status = foward_command_complete(frontend, p1, len1);
+			status = forward_command_complete(frontend, p1, len1);
 		else
-			status = foward_empty_query(frontend, p1, len1);
+			status = forward_empty_query(frontend, p1, len1);
 
 		if (status < 0)
 			return POOL_END;
@@ -179,10 +180,10 @@ CommandComplete(POOL_CONNECTION * frontend, POOL_CONNECTION_POOL * backend, bool
 		}
 
 		/*
-		 * If we are in streaming replication mode and we are doing extended
-		 * query, register query cache now.
+		 * If we are doing extended query, register the SELECT result to temp
+		 * query cache now.
 		 */
-		if (SL_MODE && pool_is_doing_extended_query_message())
+		if (pool_is_doing_extended_query_message())
 		{
 			char	   *query;
 			Node	   *node;
@@ -190,12 +191,12 @@ CommandComplete(POOL_CONNECTION * frontend, POOL_CONNECTION_POOL * backend, bool
 
 			if (session_context->query_context == NULL)
 			{
-				elog(WARNING, "expected query_contex is not NULL");
+				elog(WARNING, "expected query_contex is NULL");
 				return POOL_END;
 			}
 			query = session_context->query_context->query_w_hex;
 			node = pool_get_parse_tree();
-			state = TSTATE(backend, MASTER_NODE_ID);
+			state = TSTATE(backend, MAIN_NODE_ID);
 			pool_handle_query_cache(backend, query, node, state);
 		}
 	}
@@ -209,7 +210,7 @@ CommandComplete(POOL_CONNECTION * frontend, POOL_CONNECTION_POOL * backend, bool
 
 	/*
 	 * If we are in streaming replication mode and we are doing extended
-	 * query, reset query in progress flag and prevoius pending message.
+	 * query, reset query in progress flag and previous pending message.
 	 */
 	if (SL_MODE && pool_is_doing_extended_query_message())
 	{
@@ -247,7 +248,7 @@ CommandComplete(POOL_CONNECTION * frontend, POOL_CONNECTION_POOL * backend, bool
 }
 
 /*
- * Handle misc process which is neccessary when query context exists.
+ * Handle misc process which is necessary when query context exists.
  */
 void
 handle_query_context(POOL_CONNECTION_POOL * backend)
@@ -332,12 +333,12 @@ handle_query_context(POOL_CONNECTION_POOL * backend)
 		}
 		else if (stmt->kind == 	TRANS_STMT_COMMIT)
 		{
-			/* Commit ongoing CRETAE/DROP temp table status */
+			/* Commit ongoing CREATE/DROP temp table status */
 			pool_temp_tables_commit_pending();			
 		}
 		else if (stmt->kind == TRANS_STMT_ROLLBACK)
 		{
-			/* Remove ongoing CRETAE/DROP temp table status */
+			/* Remove ongoing CREATE/DROP temp table status */
 			pool_temp_tables_remove_pending();
 		}
 	}
@@ -349,7 +350,7 @@ handle_query_context(POOL_CONNECTION_POOL * backend)
 		/* Is this a temporary table? */
 		if (stmt->relation->relpersistence == 't')
 		{
-			if (TSTATE(backend, MASTER_NODE_ID ) == 'T')	/* Are we inside a transaction? */
+			if (TSTATE(backend, MAIN_NODE_ID ) == 'T')	/* Are we inside a transaction? */
 			{
 				state = TEMP_TABLE_CREATING;
 			}
@@ -374,7 +375,7 @@ handle_query_context(POOL_CONNECTION_POOL * backend)
 			ListCell   *cell;
 			ListCell   *next;
 
-			if (TSTATE(backend, MASTER_NODE_ID ) == 'T')	/* Are we inside a transaction? */
+			if (TSTATE(backend, MAIN_NODE_ID ) == 'T')	/* Are we inside a transaction? */
 			{
 				state = TEMP_TABLE_DROPPING;
 			}
@@ -389,7 +390,11 @@ handle_query_context(POOL_CONNECTION_POOL * backend)
 				ereport(DEBUG1,
 						(errmsg("Dropping temp table: %s", tablename)));
 				pool_temp_tables_delete(tablename, state);
-				next = lnext(cell);
+
+				if (!session_context->temp_tables)
+					return;
+
+				next = lnext(session_context->temp_tables, cell);
 			}
 		}
 	}
@@ -435,14 +440,14 @@ static POOL_STATUS handle_mismatch_tuples(POOL_CONNECTION * frontend, POOL_CONNE
 	rows = extract_ntuples(packet);
 
 	/*
-	 * Save number of affected tuples of master node.
+	 * Save number of affected tuples of main node.
 	 */
-	session_context->ntuples[MASTER_NODE_ID] = rows;
+	session_context->ntuples[MAIN_NODE_ID] = rows;
 
 
 	for (i = 0; i < NUM_BACKENDS; i++)
 	{
-		if (!IS_MASTER_NODE_ID(i))
+		if (!IS_MAIN_NODE_ID(i))
 		{
 			if (!VALID_BACKEND(i))
 			{
@@ -463,7 +468,7 @@ static POOL_STATUS handle_mismatch_tuples(POOL_CONNECTION * frontend, POOL_CONNE
 			{
 				ereport(DEBUG1,
 						(errmsg("processing command complete"),
-						 errdetail("length does not match between backends master(%d) %d th backend(%d)",
+						 errdetail("length does not match between backends main(%d) %d th backend(%d)",
 								   len, i, packetlen)));
 			}
 
@@ -495,7 +500,7 @@ static POOL_STATUS handle_mismatch_tuples(POOL_CONNECTION * frontend, POOL_CONNE
 		string_append_char(msg, "\"");
 		pool_send_error_message(frontend, MAJOR(backend),
 								"XX001", msg->data, "",
-								"check data consistency between master and other db node", __FILE__, __LINE__);
+								"check data consistency between main and other db node", __FILE__, __LINE__);
 		ereport(LOG,
 				(errmsg("%s", msg->data)));
 		free_string(msg);
@@ -517,12 +522,12 @@ static POOL_STATUS handle_mismatch_tuples(POOL_CONNECTION * frontend, POOL_CONNE
 	{
 		if (command_complete)
 		{
-			if (foward_command_complete(frontend, packet, packetlen) < 0)
+			if (forward_command_complete(frontend, packet, packetlen) < 0)
 				return POOL_END;
 		}
 		else
 		{
-			if (foward_empty_query(frontend, packet, packetlen) < 0)
+			if (forward_empty_query(frontend, packet, packetlen) < 0)
 				return POOL_END;
 		}
 	}
@@ -534,25 +539,25 @@ static POOL_STATUS handle_mismatch_tuples(POOL_CONNECTION * frontend, POOL_CONNE
  * Forward Command complete packet to frontend
  */
 static int
-foward_command_complete(POOL_CONNECTION * frontend, char *packet, int packetlen)
+forward_command_complete(POOL_CONNECTION * frontend, char *packet, int packetlen)
 {
-	return foward_packet_to_frontend(frontend, 'C', packet, packetlen);
+	return forward_packet_to_frontend(frontend, 'C', packet, packetlen);
 }
 
 /*
  * Forward Empty query response to frontend
  */
 static int
-foward_empty_query(POOL_CONNECTION * frontend, char *packet, int packetlen)
+forward_empty_query(POOL_CONNECTION * frontend, char *packet, int packetlen)
 {
-	return foward_packet_to_frontend(frontend, 'I', packet, packetlen);
+	return forward_packet_to_frontend(frontend, 'I', packet, packetlen);
 }
 
 /*
  * Forward packet to frontend
  */
 static int
-foward_packet_to_frontend(POOL_CONNECTION * frontend, char kind, char *packet, int packetlen)
+forward_packet_to_frontend(POOL_CONNECTION * frontend, char kind, char *packet, int packetlen)
 {
 	int			sendlen;
 
