@@ -5,7 +5,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2019	PgPool Global Development Group
+ * Copyright (c) 2003-2020	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -34,7 +34,7 @@
 #include <netinet/tcp.h>
 #endif
 #include <netdb.h>
-
+#include <time.h>
 #include <stdio.h>
 #include <errno.h>
 #include <signal.h>
@@ -48,6 +48,12 @@
 #include "pool_config.h"
 #include "utils/elog.h"
 #include "utils/memutils.h"
+#include "protocol/pool_connection_pool.h"
+#include "protocol/pool_process_query.h"
+#include "protocol/pool_pg_utils.h"
+#include "main/pool_internal_comms.h"
+
+
 #include "context/pool_process_context.h"
 
 static int	pool_index;			/* Active pool index */
@@ -109,18 +115,18 @@ pool_get_cp(char *user, char *database, int protoMajor, int check_socket)
 
 	for (i = 0; i < pool_config->max_pool; i++)
 	{
-		if (MASTER_CONNECTION(connection_pool) &&
-			MASTER_CONNECTION(connection_pool)->sp &&
-			MASTER_CONNECTION(connection_pool)->sp->major == protoMajor &&
-			MASTER_CONNECTION(connection_pool)->sp->user != NULL &&
-			strcmp(MASTER_CONNECTION(connection_pool)->sp->user, user) == 0 &&
-			strcmp(MASTER_CONNECTION(connection_pool)->sp->database, database) == 0)
+		if (MAIN_CONNECTION(connection_pool) &&
+			MAIN_CONNECTION(connection_pool)->sp &&
+			MAIN_CONNECTION(connection_pool)->sp->major == protoMajor &&
+			MAIN_CONNECTION(connection_pool)->sp->user != NULL &&
+			strcmp(MAIN_CONNECTION(connection_pool)->sp->user, user) == 0 &&
+			strcmp(MAIN_CONNECTION(connection_pool)->sp->database, database) == 0)
 		{
 			int			sock_broken = 0;
 			int			j;
 
 			/* mark this connection is under use */
-			MASTER_CONNECTION(connection_pool)->closetime = 0;
+			MAIN_CONNECTION(connection_pool)->closetime = 0;
 			for (j = 0; j < NUM_BACKENDS; j++)
 			{
 				connection_pool->info[j].counter++;
@@ -253,7 +259,7 @@ pool_create_cp(void)
 
 	for (i = 0; i < pool_config->max_pool; i++)
 	{
-		if (MASTER_CONNECTION(p) == NULL)
+		if (MAIN_CONNECTION(p) == NULL)
 		{
 			ret = new_connection(p);
 			if (ret)
@@ -271,7 +277,7 @@ pool_create_cp(void)
 	 * discard it.
 	 */
 	oldestp = p = pool_connection_pool;
-	closetime = MASTER_CONNECTION(p)->closetime;
+	closetime = MAIN_CONNECTION(p)->closetime;
 	pool_index = 0;
 
 	for (i = 0; i < pool_config->max_pool; i++)
@@ -279,13 +285,13 @@ pool_create_cp(void)
 		ereport(DEBUG1,
 				(errmsg("creating connection pool"),
 				 errdetail("user: %s database: %s closetime: %ld",
-						   MASTER_CONNECTION(p)->sp->user,
-						   MASTER_CONNECTION(p)->sp->database,
-						   MASTER_CONNECTION(p)->closetime)));
+						   MAIN_CONNECTION(p)->sp->user,
+						   MAIN_CONNECTION(p)->sp->database,
+						   MAIN_CONNECTION(p)->closetime)));
 
-		if (MASTER_CONNECTION(p)->closetime < closetime)
+		if (MAIN_CONNECTION(p)->closetime < closetime)
 		{
-			closetime = MASTER_CONNECTION(p)->closetime;
+			closetime = MAIN_CONNECTION(p)->closetime;
 			oldestp = p;
 			pool_index = i;
 		}
@@ -299,8 +305,8 @@ pool_create_cp(void)
 			(errmsg("creating connection pool"),
 			 errdetail("discarding old %zd th connection. user: %s database: %s",
 					   oldestp - pool_connection_pool,
-					   MASTER_CONNECTION(p)->sp->user,
-					   MASTER_CONNECTION(p)->sp->database)));
+					   MAIN_CONNECTION(p)->sp->user,
+					   MAIN_CONNECTION(p)->sp->database)));
 
 	for (i = 0; i < NUM_BACKENDS; i++)
 	{
@@ -354,14 +360,14 @@ pool_connection_pool_timer(POOL_CONNECTION_POOL * backend)
 	/* look for any other timeout */
 	for (i = 0; i < pool_config->max_pool; i++, p++)
 	{
-		if (!MASTER_CONNECTION(p))
+		if (!MAIN_CONNECTION(p))
 			continue;
-		if (!MASTER_CONNECTION(p)->sp)
+		if (!MAIN_CONNECTION(p)->sp)
 			continue;
-		if (MASTER_CONNECTION(p)->sp->user == NULL)
+		if (MAIN_CONNECTION(p)->sp->user == NULL)
 			continue;
 
-		if (p != backend && MASTER_CONNECTION(p)->closetime)
+		if (p != backend && MAIN_CONNECTION(p)->closetime)
 			return;
 	}
 
@@ -403,30 +409,30 @@ pool_backend_timer(void)
 
 	for (i = 0; i < pool_config->max_pool; i++, p++)
 	{
-		if (!MASTER_CONNECTION(p))
+		if (!MAIN_CONNECTION(p))
 			continue;
-		if (!MASTER_CONNECTION(p)->sp)
+		if (!MAIN_CONNECTION(p)->sp)
 			continue;
-		if (MASTER_CONNECTION(p)->sp->user == NULL)
+		if (MAIN_CONNECTION(p)->sp->user == NULL)
 			continue;
 
 		/* timer expire? */
-		if (MASTER_CONNECTION(p)->closetime)
+		if (MAIN_CONNECTION(p)->closetime)
 		{
 			int			freed = 0;
 
 			ereport(DEBUG1,
 					(errmsg("backend timer handler called"),
 					 errdetail("expire time: %ld",
-							   MASTER_CONNECTION(p)->closetime + pool_config->connection_life_time)));
+							   MAIN_CONNECTION(p)->closetime + pool_config->connection_life_time)));
 
-			if (now >= (MASTER_CONNECTION(p)->closetime + pool_config->connection_life_time))
+			if (now >= (MAIN_CONNECTION(p)->closetime + pool_config->connection_life_time))
 			{
 				/* discard expired connection */
 				ereport(DEBUG1,
 						(errmsg("backend timer handler called"),
 						 errdetail("expired user: \"%s\" database: \"%s\"",
-								   MASTER_CONNECTION(p)->sp->user, MASTER_CONNECTION(p)->sp->database)));
+								   MAIN_CONNECTION(p)->sp->user, MAIN_CONNECTION(p)->sp->database)));
 				pool_send_frontend_exits(p);
 
 				for (j = 0; j < NUM_BACKENDS; j++)
@@ -451,8 +457,8 @@ pool_backend_timer(void)
 			else
 			{
 				/* look for nearest timer */
-				if (MASTER_CONNECTION(p)->closetime < nearest)
-					nearest = MASTER_CONNECTION(p)->closetime;
+				if (MAIN_CONNECTION(p)->closetime < nearest)
+					nearest = MAIN_CONNECTION(p)->closetime;
 			}
 		}
 	}
@@ -515,7 +521,7 @@ connect_unix_domain_socket_by_port(int port, char *socket_dir, bool retry)
 	{
 		ereport(LOG,
 				(errmsg("failed to connect to PostgreSQL server by unix domain socket"),
-				 errdetail("create socket failed with error \"%s\"", strerror(errno))));
+				 errdetail("create socket failed with error \"%m\"")));
 		return -1;
 	}
 
@@ -542,7 +548,7 @@ connect_unix_domain_socket_by_port(int port, char *socket_dir, bool retry)
 			close(fd);
 			ereport(LOG,
 					(errmsg("failed to connect to PostgreSQL server by unix domain socket"),
-					 errdetail("connect to \"%s\" failed with error \"%s\"", addr.sun_path, strerror(errno))));
+					 errdetail("connect to \"%s\" failed with error \"%m\"", addr.sun_path)));
 
 			return -1;
 		}
@@ -553,7 +559,7 @@ connect_unix_domain_socket_by_port(int port, char *socket_dir, bool retry)
 }
 
 /*
- * Connet to backend using pool_config->connect_timeout.
+ * Connect to backend using pool_config->connect_timeout.
  *
  * fd: the socket
  * walk: backend address to connect
@@ -572,7 +578,7 @@ connect_with_timeout(int fd, struct addrinfo *walk, char *host, int port, bool r
 	int			error;
 	socklen_t	socklen;
 
-	pool_set_nonblock(fd);
+	socket_set_nonblock(fd);
 
 	for (;;)
 	{
@@ -612,7 +618,8 @@ connect_with_timeout(int fd, struct addrinfo *walk, char *host, int port, bool r
 			if (errno != EINPROGRESS && errno != EALREADY)
 			{
 				ereport(LOG,
-						(errmsg("failed to connect to PostgreSQL server on \"%s:%d\" with error \"%s\"", host, port, strerror(errno))));
+						(errmsg("failed to connect to PostgreSQL server on \"%s:%d\"", host, port),
+						 errdetail("%m")));
 				return false;
 			}
 
@@ -661,7 +668,7 @@ connect_with_timeout(int fd, struct addrinfo *walk, char *host, int port, bool r
 				 * If read data or write data was set, either connect
 				 * succeeded or error.  We need to figure it out. This is the
 				 * hardest part in using non blocking connect(2).  See W.
-				 * Richar Stevens's "UNIX Network Programming: Volume 1,
+				 * Richard Stevens's "UNIX Network Programming: Volume 1,
 				 * Second Edition" section 15.4.
 				 */
 				if (FD_ISSET(fd, &rset) || FD_ISSET(fd, &wset))
@@ -672,7 +679,8 @@ connect_with_timeout(int fd, struct addrinfo *walk, char *host, int port, bool r
 					{
 						/* Solaris returns error in this case */
 						ereport(LOG,
-								(errmsg("failed to connect to PostgreSQL server on \"%s:%d\", getsockopt() failed with error \"%s\"", host, port, strerror(errno))));
+								(errmsg("failed to connect to PostgreSQL server on \"%s:%d\", getsockopt() failed", host, port),
+								 errdetail("%m")));
 
 						return false;
 					}
@@ -681,7 +689,8 @@ connect_with_timeout(int fd, struct addrinfo *walk, char *host, int port, bool r
 					if (error != 0)
 					{
 						ereport(LOG,
-								(errmsg("failed to connect to PostgreSQL server on \"%s:%d\", getsockopt() detected error \"%s\"", host, port, strerror(error))));
+								(errmsg("failed to connect to PostgreSQL server on \"%s:%d\", getsockopt() failed", host, port),
+								 errdetail("%m")));
 						return false;
 					}
 				}
@@ -728,7 +737,7 @@ connect_with_timeout(int fd, struct addrinfo *walk, char *host, int port, bool r
 				{
 					ereport(LOG,
 							(errmsg("failed to connect to PostgreSQL server on \"%s:%d\" using INET socket", host, port),
-							 errdetail("select() system call failed with an error \"%s\"", strerror(errno))));
+							 errdetail("select() system call failed with error \"%m\"")));
 				}
 				close(fd);
 				return false;
@@ -737,7 +746,7 @@ connect_with_timeout(int fd, struct addrinfo *walk, char *host, int port, bool r
 		break;
 	}
 
-	pool_unset_nonblock(fd);
+	socket_unset_nonblock(fd);
 	return true;
 }
 
@@ -763,7 +772,8 @@ connect_inet_domain_socket_by_port(char *host, int port, bool retry)
 	if (asprintf(&portstr, "%d", port) == -1)
 	{
 		ereport(WARNING,
-				(errmsg("failed to connect to PostgreSQL server, asprintf() failed with error \"%s\"", strerror(errno))));
+				(errmsg("failed to connect to PostgreSQL server, asprintf() failed"),
+				 errdetail("%m")));
 
 		return -1;
 	}
@@ -789,7 +799,8 @@ connect_inet_domain_socket_by_port(char *host, int port, bool retry)
 		if (fd < 0)
 		{
 			ereport(WARNING,
-					(errmsg("failed to connect to PostgreSQL server, socket() failed with error \"%s\"", strerror(errno))));
+					(errmsg("failed to connect to PostgreSQL server, socket() failed"),
+					 errdetail("%m")));
 			continue;
 		}
 
@@ -799,7 +810,8 @@ connect_inet_domain_socket_by_port(char *host, int port, bool retry)
 					   sizeof(on)) < 0)
 		{
 			ereport(WARNING,
-					(errmsg("failed to connect to PostgreSQL server, setsockopt() failed with error \"%s\"", strerror(errno))));
+					(errmsg("failed to connect to PostgreSQL server, setsockopt() failed"),
+					 errdetail("%m")));
 
 			close(fd);
 			freeaddrinfo(res);
@@ -925,16 +937,16 @@ static POOL_CONNECTION_POOL * new_connection(POOL_CONNECTION_POOL * p)
 					/* set down status to local status area */
 					*(my_backend_status[i]) = CON_DOWN;
 
-					/* if master_node_id is not updated, then update it */
-					if (Req_info->master_node_id == i)
+					/* if main_node_id is not updated, then update it */
+					if (Req_info->main_node_id == i)
 					{
-						int			old_master = Req_info->master_node_id;
+						int			old_main = Req_info->main_node_id;
 
-						Req_info->master_node_id = get_next_master_node();
+						Req_info->main_node_id = get_next_main_node();
 
 						ereport(LOG,
-								(errmsg("master node %d is down. Update master node to %d",
-										old_master, Req_info->master_node_id)));
+								(errmsg("main node %d is down. Update main node to %d",
+										old_main, Req_info->main_node_id)));
 					}
 
 					/*
@@ -956,6 +968,7 @@ static POOL_CONNECTION_POOL * new_connection(POOL_CONNECTION_POOL * p)
 		else
 		{
 			p->info[i].create_time = time(NULL);
+			p->info[i].client_idle_duration = 0;
 			p->slots[i] = s;
 
 			pool_init_params(&s->con->params);
@@ -1023,3 +1036,33 @@ pool_pool_index(void)
 {
 	return pool_index;
 }
+
+/*
+ * send frontend exiting messages to all connections.  this is called
+ * in any case when child process exits, for example failover, child
+ * life time expires or child max connections expires.
+ */
+
+void
+close_all_backend_connections(void)
+{
+	int			i;
+	POOL_CONNECTION_POOL *p = pool_connection_pool;
+
+	pool_sigset_t oldmask;
+
+	POOL_SETMASK2(&BlockSig, &oldmask);
+
+	for (i = 0; i < pool_config->max_pool; i++, p++)
+	{
+		if (!MAIN_CONNECTION(p))
+			continue;
+		if (!MAIN_CONNECTION(p)->sp)
+			continue;
+		if (MAIN_CONNECTION(p)->sp->user == NULL)
+			continue;
+		pool_send_frontend_exits(p);
+	}
+
+	POOL_SETMASK(&oldmask);
+}
