@@ -2,7 +2,7 @@
 /*
  * $Header$
  *
- * Copyright (c) 2003-2012	PgPool Global Development Group
+ * Copyright (c) 2003-2022	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -16,24 +16,25 @@
  * is" without express or implied warranty.
  *
  * pgpool-regclass.c is similar to PostgreSQL builtin function
- * reglcass but does not throw exceptions.
+ * regclass but does not throw exceptions.
  * If something goes wrong, it returns InvalidOid.
  */
 
+#include <stdlib.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
+
 #include "postgres.h"
-#include "utils/builtins.h"
-#include "utils/syscache.h"
-#include "utils/elog.h"
 #include "catalog/namespace.h"
-#include "nodes/makefuncs.h"
 #include "commands/dbcommands.h"
 #include "fmgr.h"
 #include "funcapi.h"
-
-#include <stdlib.h>
+#include "nodes/makefuncs.h"
+#include "utils/builtins.h"
+#include "utils/elog.h"
+#include "utils/syscache.h"
+#include "utils/varlena.h"
 
 #ifdef PG_MODULE_MAGIC
 PG_MODULE_MAGIC;
@@ -97,7 +98,7 @@ pgpool_regclass(PG_FUNCTION_ARGS)
 
 	/*
 	 * RangeVarGetRelid() of PostgreSQL 9.2 or later, has third argument
-	 * "missing_ok" which suppresses ERROR exception, but returns invlaid_oid.
+	 * "missing_ok" which suppresses ERROR exception, but returns invalid_oid.
 	 * See include/catalog/namespace.h
 	 */
 	result = RangeVarGetRelid(rel, true, true);
