@@ -20,12 +20,14 @@ PG_MODULE_MAGIC;
 
 
 Datum		_pcp_node_info(PG_FUNCTION_ARGS);
+Datum		_pcp_health_check_stats(PG_FUNCTION_ARGS);
 Datum		_pcp_pool_status(PG_FUNCTION_ARGS);
 Datum		_pcp_node_count(PG_FUNCTION_ARGS);
 Datum		_pcp_attach_node(PG_FUNCTION_ARGS);
 Datum		_pcp_detach_node(PG_FUNCTION_ARGS);
 
 PG_FUNCTION_INFO_V1(_pcp_node_info);
+PG_FUNCTION_INFO_V1(_pcp_health_check_stats);
 PG_FUNCTION_INFO_V1(_pcp_pool_status);
 PG_FUNCTION_INFO_V1(_pcp_node_count);
 PG_FUNCTION_INFO_V1(_pcp_attach_node);
