@@ -4,7 +4,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2019	PgPool Global Development Group
+ * Copyright (c) 2003-2021	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -43,11 +43,13 @@ char	   *last_dir_separator(const char *filename);
 
 static void usage(void);
 static inline bool app_require_nodeID(void);
+static inline bool app_support_cluster_mode(void);
 static void output_watchdog_info_result(PCPResultInfo * pcpResInfo, bool verbose);
 static void output_procinfo_result(PCPResultInfo * pcpResInfo, bool all, bool verbose);
 static void output_proccount_result(PCPResultInfo * pcpResInfo, bool verbose);
 static void output_poolstatus_result(PCPResultInfo * pcpResInfo, bool verbose);
-static void output_nodeinfo_result(PCPResultInfo * pcpResInfo, bool verbose);
+static void output_nodeinfo_result(PCPResultInfo * pcpResInfo, bool all,  bool verbose);
+static void output_health_check_stats_result(PCPResultInfo * pcpResInfo, bool verbose);
 static void output_nodecount_result(PCPResultInfo * pcpResInfo, bool verbose);
 static char *backend_status_to_string(BackendInfo * bi);
 static char *format_titles(const char **titles, const char **types, int ntitles);
@@ -58,6 +60,7 @@ typedef enum
 	PCP_DETACH_NODE,
 	PCP_NODE_COUNT,
 	PCP_NODE_INFO,
+	PCP_HEALTH_CHECK_STATS,
 	PCP_POOL_STATUS,
 	PCP_PROC_COUNT,
 	PCP_PROC_INFO,
@@ -65,6 +68,7 @@ typedef enum
 	PCP_RECOVERY_NODE,
 	PCP_STOP_PGPOOL,
 	PCP_WATCHDOG_INFO,
+	PCP_RELOAD_CONFIG,
 	UNKNOWN,
 }			PCP_UTILITIES;
 
@@ -81,14 +85,16 @@ struct AppTypes AllAppTypes[] =
 	{"pcp_attach_node", PCP_ATTACH_NODE, "n:h:p:U:wWvd", "attach a node from pgpool-II"},
 	{"pcp_detach_node", PCP_DETACH_NODE, "n:h:p:U:gwWvd", "detach a node from pgpool-II"},
 	{"pcp_node_count", PCP_NODE_COUNT, "h:p:U:wWvd", "display the total number of nodes under pgpool-II's control"},
-	{"pcp_node_info", PCP_NODE_INFO, "n:h:p:U:wWvd", "display a pgpool-II node's information"},
+	{"pcp_node_info", PCP_NODE_INFO, "n:h:p:U:awWvd", "display a pgpool-II node's information"},
+	{"pcp_health_check_stats", PCP_HEALTH_CHECK_STATS, "n:h:p:U:wWvd", "display a pgpool-II health check stats data"},
 	{"pcp_pool_status", PCP_POOL_STATUS, "h:p:U:wWvd", "display pgpool configuration and status"},
 	{"pcp_proc_count", PCP_PROC_COUNT, "h:p:U:wWvd", "display the list of pgpool-II child process PIDs"},
 	{"pcp_proc_info", PCP_PROC_INFO, "h:p:P:U:awWvd", "display a pgpool-II child process' information"},
-	{"pcp_promote_node", PCP_PROMOTE_NODE, "n:h:p:U:gwWvd", "promote a node as new master from pgpool-II"},
+	{"pcp_promote_node", PCP_PROMOTE_NODE, "n:h:p:U:gswWvd", "promote a node as new main from pgpool-II"},
 	{"pcp_recovery_node", PCP_RECOVERY_NODE, "n:h:p:U:wWvd", "recover a node"},
-	{"pcp_stop_pgpool", PCP_STOP_PGPOOL, "m:h:p:U:wWvd", "terminate pgpool-II"},
+	{"pcp_stop_pgpool", PCP_STOP_PGPOOL, "m:h:p:U:s:wWvda", "terminate pgpool-II"},
 	{"pcp_watchdog_info", PCP_WATCHDOG_INFO, "n:h:p:U:wWvd", "display a pgpool-II watchdog's information"},
+	{"pcp_reload_config",PCP_RELOAD_CONFIG,"h:p:U:s:wWvd", "reload a pgpool-II config file"},
 	{NULL, UNKNOWN, NULL, NULL},
 };
 struct AppTypes *current_app_type;
@@ -104,12 +110,14 @@ main(int argc, char **argv)
 	int			processID = 0;
 	int			ch;
 	char		shutdown_mode = 's';
+	char		command_scope = 'l';
 	int			optindex;
 	int			i;
 	bool		all = false;
 	bool		debug = false;
 	bool		need_password = true;
 	bool		gracefully = false;
+	bool		switchover = false;
 	bool		verbose = false;
 	PCPConnInfo *pcpConn;
 	PCPResultInfo *pcpResInfo;
@@ -126,7 +134,9 @@ main(int argc, char **argv)
 		{"no-password", no_argument, NULL, 'w'},
 		{"password", no_argument, NULL, 'W'},
 		{"mode", required_argument, NULL, 'm'},
+		{"scope", required_argument, NULL, 's'},
 		{"gracefully", no_argument, NULL, 'g'},
+		{"switchover", no_argument, NULL, 's'},
 		{"verbose", no_argument, NULL, 'v'},
 		{"all", no_argument, NULL, 'a'},
 		{"node-id", required_argument, NULL, 'n'},
@@ -190,6 +200,33 @@ main(int argc, char **argv)
 				gracefully = true;
 				break;
 
+			case 's':
+				if (app_support_cluster_mode())
+				{
+					if (strcmp(optarg, "c") == 0 || strcmp(optarg, "cluster") == 0)
+						command_scope = 'c';
+					else if (strcmp(optarg, "l") == 0 || strcmp(optarg, "local") == 0)
+						command_scope = 'l';
+					else
+					{
+						fprintf(stderr, "%s: Invalid command scope \"%s\", must be either \"cluster\" or \"local\" \n", progname, optarg);
+						exit(1);
+					}
+				}
+				else
+				{
+					if (current_app_type->app_type == PCP_PROMOTE_NODE)
+					{
+						switchover = true;
+					}
+					else
+					{
+						fprintf(stderr, "Invalid argument \"%s\", Try \"%s --help\" for more information.\n", optarg, progname);
+						exit(1);
+					}
+				}
+				break;
+
 			case 'm':
 				if (current_app_type->app_type == PCP_STOP_PGPOOL)
 				{
@@ -266,7 +303,7 @@ main(int argc, char **argv)
 			default:
 
 				/*
-				 * getopt_long whould already have emitted a complaint
+				 * getopt_long should already have emitted a complaint
 				 */
 				fprintf(stderr, "Try \"%s --help\" for more information.\n\n", progname);
 				exit(1);
@@ -296,7 +333,7 @@ main(int argc, char **argv)
 				exit(0);
 			}
 		}
-		else if (app_require_nodeID() && nodeID < 0)
+		else if ((app_require_nodeID() || current_app_type->app_type == PCP_NODE_INFO) && nodeID < 0)
 		{
 			nodeID = atoi(argv[optind]);
 			if (nodeID < 0 || nodeID > MAX_NUM_BACKENDS)
@@ -320,7 +357,7 @@ main(int argc, char **argv)
 			fprintf(stderr, "Try \"%s --help\" for more information.\n\n", progname);
 			exit(1);
 		}
-		else if (current_app_type->app_type == PCP_WATCHDOG_INFO)
+		else if (current_app_type->app_type == PCP_WATCHDOG_INFO || current_app_type->app_type == PCP_NODE_INFO)
 		{
 			nodeID = -1;
 		}
@@ -366,6 +403,11 @@ main(int argc, char **argv)
 		pcpResInfo = pcp_node_info(pcpConn, nodeID);
 	}
 
+	else if (current_app_type->app_type == PCP_HEALTH_CHECK_STATS)
+	{
+		pcpResInfo = pcp_health_check_stats(pcpConn, nodeID);
+	}
+
 	else if (current_app_type->app_type == PCP_POOL_STATUS)
 	{
 		pcpResInfo = pcp_pool_status(pcpConn);
@@ -384,9 +426,9 @@ main(int argc, char **argv)
 	else if (current_app_type->app_type == PCP_PROMOTE_NODE)
 	{
 		if (gracefully)
-			pcpResInfo = pcp_promote_node_gracefully(pcpConn, nodeID);
+			pcpResInfo = pcp_promote_node_gracefully(pcpConn, nodeID, switchover);
 		else
-			pcpResInfo = pcp_promote_node(pcpConn, nodeID);
+			pcpResInfo = pcp_promote_node(pcpConn, nodeID, switchover);
 	}
 
 	else if (current_app_type->app_type == PCP_RECOVERY_NODE)
@@ -396,7 +438,7 @@ main(int argc, char **argv)
 
 	else if (current_app_type->app_type == PCP_STOP_PGPOOL)
 	{
-		pcpResInfo = pcp_terminate_pgpool(pcpConn, shutdown_mode);
+		pcpResInfo = pcp_terminate_pgpool(pcpConn, shutdown_mode, command_scope);
 	}
 
 	else if (current_app_type->app_type == PCP_WATCHDOG_INFO)
@@ -404,6 +446,11 @@ main(int argc, char **argv)
 		pcpResInfo = pcp_watchdog_info(pcpConn, nodeID);
 	}
 
+	else if (current_app_type->app_type == PCP_RELOAD_CONFIG)
+	{
+		pcpResInfo = pcp_reload_config(pcpConn,command_scope);
+	}
+
 	else
 	{
 		/* should never happen */
@@ -427,7 +474,10 @@ main(int argc, char **argv)
 			output_nodecount_result(pcpResInfo, verbose);
 
 		if (current_app_type->app_type == PCP_NODE_INFO)
-			output_nodeinfo_result(pcpResInfo, verbose);
+			output_nodeinfo_result(pcpResInfo, all, verbose);
+
+		if (current_app_type->app_type == PCP_HEALTH_CHECK_STATS)
+			output_health_check_stats_result(pcpResInfo, verbose);
 
 		if (current_app_type->app_type == PCP_POOL_STATUS)
 			output_poolstatus_result(pcpResInfo, verbose);
@@ -464,47 +514,126 @@ output_nodecount_result(PCPResultInfo * pcpResInfo, bool verbose)
 }
 
 static void
-output_nodeinfo_result(PCPResultInfo * pcpResInfo, bool verbose)
+output_nodeinfo_result(PCPResultInfo * pcpResInfo, bool all, bool verbose)
 {
-	BackendInfo *backend_info = (BackendInfo *) pcp_get_binary_data(pcpResInfo, 0);
+	bool		printed = false;
+	int			i;
 	char		last_status_change[20];
 	struct tm	tm;
-
-	localtime_r(&backend_info->status_changed_time, &tm);
-	strftime(last_status_change, sizeof(last_status_change), "%F %T", &tm);
+	char	   *frmt;
+	int         array_size = pcp_result_slot_count(pcpResInfo);
 
 	if (verbose)
 	{
-		const char *titles[] = {"Hostname", "Port", "Status", "Weight", "Status Name", "Role", "Replication Delay", "Replication State", "Replication Sync State", "Last Status Change"};
-		const char *types[] = {"s", "d", "d", "f", "s", "s", "lu", "s", "s", "s"};
+		const char *titles[] = {"Hostname", "Port", "Status", "Weight", "Status Name", "Backend Status Name", "Role", "Backend Role", "Replication Delay", "Replication State", "Replication Sync State", "Last Status Change"};
+		const char *types[] = {"s", "d", "d", "f", "s", "s", "s", "s", "lu", "s", "s", "s"};
+
+		frmt = format_titles(titles, types, sizeof(titles)/sizeof(char *));
+	}
+	else
+	{
+		frmt = "%s %d %d %f %s %s %s %s %lu %s %s %s\n";
+	}
+
+	for (i = 0; i < array_size; i++)
+	{
+
+		BackendInfo *backend_info = (BackendInfo *) pcp_get_binary_data(pcpResInfo, i);
+
+		if (backend_info == NULL)
+			break;
+		if ((!all) && (backend_info->backend_hostname[0] == '\0'))
+			continue;
+
+		printed = true;
+		localtime_r(&backend_info->status_changed_time, &tm);
+		strftime(last_status_change, sizeof(last_status_change), "%F %T", &tm);
+
+		printf(frmt,
+			   backend_info->backend_hostname,
+			   backend_info->backend_port,
+			   backend_info->backend_status,
+			   backend_info->backend_weight / RAND_MAX,
+			   backend_status_to_string(backend_info),
+			   backend_info->pg_backend_status,
+			   role_to_str(backend_info->role),
+			   backend_info->pg_role,
+			   backend_info->standby_delay,
+			   backend_info->replication_state[0] == '\0' ? "none" : backend_info->replication_state,
+			   backend_info->replication_sync_state[0] == '\0' ? "none" : backend_info->replication_sync_state,
+			   last_status_change);
+	}
+
+	if (printed == false)
+		printf("No node information available\n\n");
+}
+
+/*
+ * Format and output health check stats
+ */
+static void
+output_health_check_stats_result(PCPResultInfo * pcpResInfo, bool verbose)
+{
+	POOL_HEALTH_CHECK_STATS *stats = (POOL_HEALTH_CHECK_STATS *)pcp_get_binary_data(pcpResInfo, 0);
+
+	if (verbose)
+	{
+		const char *titles[] = {"Node Id", "Host Name", "Port", "Status", "Role", "Last Status Change",
+								"Total Count", "Success Count", "Fail Count", "Skip Count", "Retry Count",
+								"Average Retry Count", "Max Retry Count", "Max Health Check Duration",
+								"Minimum Health Check Duration", "Average Health Check Duration",
+								"Last Health Check", "Last Successful Health Check",
+								"Last Skip Health Check", "Last Failed Health Check"};
+		const char *types[] = {"s", "s", "s", "s", "s", "s", "s", "s", "s", "s",
+							   "s", "s", "s", "s", "s", "s", "s", "s", "s", "s"};
 		char *format_string;
 
 		format_string = format_titles(titles, types, sizeof(titles)/sizeof(char *));
 		printf(format_string,
-			   backend_info->backend_hostname,
-			   backend_info->backend_port,
-			   backend_info->backend_status,
-			   backend_info->backend_weight / RAND_MAX,
-			   backend_status_to_string(backend_info),
-			   role_to_str(backend_info->role),
-			   backend_info->standby_delay,
-			   backend_info->replication_state,
-			   backend_info->replication_sync_state,
-			   last_status_change);
+			   stats->node_id,
+			   stats->hostname,
+			   stats->port,
+			   stats->status,
+			   stats->role,
+			   stats->last_status_change,
+			   stats->total_count,
+			   stats->success_count,
+			   stats->fail_count,
+			   stats->skip_count,
+			   stats->retry_count,
+			   stats->average_retry_count,
+			   stats->max_retry_count,
+			   stats->max_health_check_duration,
+			   stats->min_health_check_duration,
+			   stats->average_health_check_duration,
+			   stats->last_health_check,
+			   stats->last_successful_health_check,
+			   stats->last_skip_health_check,
+			   stats->last_failed_health_check);
 	}
 	else
 	{
-		printf("%s %d %d %f %s %s %lu %s %s %s\n",
-			   backend_info->backend_hostname,
-			   backend_info->backend_port,
-			   backend_info->backend_status,
-			   backend_info->backend_weight / RAND_MAX,
-			   backend_status_to_string(backend_info),
-			   role_to_str(backend_info->role),
-			   backend_info->standby_delay,
-			   backend_info->replication_state,
-			   backend_info->replication_sync_state,
-			   last_status_change);
+		printf("%s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s\n",
+			   stats->node_id,
+			   stats->hostname,
+			   stats->port,
+			   stats->status,
+			   stats->role,
+			   stats->last_status_change,
+			   stats->total_count,
+			   stats->success_count,
+			   stats->fail_count,
+			   stats->skip_count,
+			   stats->retry_count,
+			   stats->average_retry_count,
+			   stats->max_retry_count,
+			   stats->max_health_check_duration,
+			   stats->min_health_check_duration,
+			   stats->average_health_check_duration,
+			   stats->last_health_check,
+			   stats->last_successful_health_check,
+			   stats->last_skip_health_check,
+			   stats->last_failed_health_check);
 	}
 }
 
@@ -568,59 +697,57 @@ output_procinfo_result(PCPResultInfo * pcpResInfo, bool all, bool verbose)
 {
 	bool		printed = false;
 	int			i;
-	char	   *frmt;
-	char		strcreatetime[128];
-	char		strstarttime[128];
+	char	   *format;
 	int			array_size = pcp_result_slot_count(pcpResInfo);
 
+	const char *titles[] = {
+		"Database", "Username", "Start time", "Client connection count",
+		"Major", "Minor", "Backend connection time", "Client connection time",
+		"Client idle duration", "Client disconnection time", "Pool Counter", "Backend PID",
+		"Connected", "PID", "Backend ID", "Status"
+	};
+	const char *types[] = {
+		"s", "s", "s", "s",
+		"s", "s", "s", "s",
+		"s", "s", "s", "s",
+		"s", "s", "s", "s"
+	};
+
+
 	if (verbose)
-	{
-		frmt = "Database     : %s\n"
-			"Username     : %s\n"
-			"Start time   : %s\n"
-			"Creation time: %s\n"
-			"Major        : %d\n"
-			"Minor        : %d\n"
-			"Counter      : %d\n"
-			"Backend PID  : %d\n"
-			"Connected    : %d\n"
-			"PID          : %d\n"
-			"Backend ID   : %d\n";
-	}
+		format = format_titles(titles, types, sizeof(titles)/sizeof(char *));
 	else
 	{
-		frmt = "%s %s %s %s %d %d %d %d %d %d %d\n";
+		format = "%s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s\n";
 	}
 
 	for (i = 0; i < array_size; i++)
 	{
 
-		ProcessInfo *process_info = (ProcessInfo *) pcp_get_binary_data(pcpResInfo, i);
+		POOL_REPORT_POOLS *pools = (POOL_REPORT_POOLS *) pcp_get_binary_data(pcpResInfo, i);
 
-		if (process_info == NULL)
+		if (pools == NULL)
 			break;
-		if ((!all) && (process_info->connection_info->database[0] == '\0'))
+		if ((!all) && (pools->database[0] == '\0'))
 			continue;
 		printed = true;
-		*strcreatetime = *strstarttime = '\0';
-
-		if (process_info->start_time)
-			strftime(strstarttime, 128, "%Y-%m-%d %H:%M:%S", localtime(&process_info->start_time));
-		if (process_info->connection_info->create_time)
-			strftime(strcreatetime, 128, "%Y-%m-%d %H:%M:%S", localtime(&process_info->connection_info->create_time));
-
-		printf(frmt,
-			   process_info->connection_info->database,
-			   process_info->connection_info->user,
-			   strstarttime,
-			   strcreatetime,
-			   process_info->connection_info->major,
-			   process_info->connection_info->minor,
-			   process_info->connection_info->counter,
-			   process_info->connection_info->pid,
-			   process_info->connection_info->connected,
-			   process_info->pid,
-			   process_info->connection_info->backend_id);
+		printf(format,
+			   pools->database,
+			   pools->username,
+			   pools->process_start_time,
+			   pools->client_connection_count,
+			   pools->pool_majorversion,
+			   pools->pool_minorversion,
+			   pools->backend_connection_time,
+			   pools->client_connection_time,
+			   pools->client_idle_duration,
+			   pools->client_disconnection_time,
+			   pools->pool_counter,
+			   pools->pool_backendpid,
+			   pools->pool_connected,
+			   pools->pool_pid,
+			   pools->backend_id,
+			   pools->status);
 	}
 	if (printed == false)
 		printf("No process information available\n\n");
@@ -643,53 +770,59 @@ output_watchdog_info_result(PCPResultInfo * pcpResInfo, bool verbose)
 		else if (cluster->quorumStatus == -1)
 			quorumStatus = "QUORUM ABSENT";
 		else if (cluster->quorumStatus == -2)
-			quorumStatus = "NO MASTER NODE";
+			quorumStatus = "NO LEADER NODE";
 		else
 			quorumStatus = "UNKNOWN";
 
 		printf("Watchdog Cluster Information \n");
-		printf("Total Nodes          : %d\n", cluster->remoteNodeCount + 1);
-		printf("Remote Nodes         : %d\n", cluster->remoteNodeCount);
-		printf("Quorum state         : %s\n", quorumStatus);
-		printf("Alive Remote Nodes   : %d\n", cluster->aliveNodeCount);
-		printf("VIP up on local node : %s\n", cluster->escalated ? "YES" : "NO");
-		printf("Master Node Name     : %s\n", cluster->masterNodeName);
-		printf("Master Host Name     : %s\n\n", cluster->masterHostName);
+		printf("Total Nodes              : %d\n", cluster->remoteNodeCount + 1);
+		printf("Remote Nodes             : %d\n", cluster->remoteNodeCount);
+		printf("Member Remote Nodes      : %d\n", cluster->memberRemoteNodeCount);
+		printf("Alive Remote Nodes       : %d\n", cluster->aliveNodeCount);
+		printf("Nodes required for quorum: %d\n", cluster->nodesRequiredForQuorum);
+		printf("Quorum state             : %s\n", quorumStatus);
+		printf("Local node escalation    : %s\n", cluster->escalated ? "YES" : "NO");
+		printf("Leader Node Name         : %s\n", cluster->leaderNodeName);
+		printf("Leader Host Name         : %s\n\n", cluster->leaderHostName);
 
 		printf("Watchdog Node Information \n");
 		for (i = 0; i < cluster->nodeCount; i++)
 		{
 			PCPWDNodeInfo *watchdog_info = &cluster->nodeList[i];
 
-			printf("Node Name      : %s\n", watchdog_info->nodeName);
-			printf("Host Name      : %s\n", watchdog_info->hostName);
-			printf("Delegate IP    : %s\n", watchdog_info->delegate_ip);
-			printf("Pgpool port    : %d\n", watchdog_info->pgpool_port);
-			printf("Watchdog port  : %d\n", watchdog_info->wd_port);
-			printf("Node priority  : %d\n", watchdog_info->wd_priority);
-			printf("Status         : %d\n", watchdog_info->state);
-			printf("Status Name    : %s\n\n", watchdog_info->stateName);
+			printf("Node Name         : %s\n", watchdog_info->nodeName);
+			printf("Host Name         : %s\n", watchdog_info->hostName);
+			printf("Delegate IP       : %s\n", watchdog_info->delegate_ip);
+			printf("Pgpool port       : %d\n", watchdog_info->pgpool_port);
+			printf("Watchdog port     : %d\n", watchdog_info->wd_port);
+			printf("Node priority     : %d\n", watchdog_info->wd_priority);
+			printf("Status            : %d\n", watchdog_info->state);
+			printf("Status Name       : %s\n", watchdog_info->stateName);
+			printf("Membership Status : %s\n\n", watchdog_info->membership_status_string);
 		}
 	}
 	else
 	{
-		printf("%d %s %s %s\n\n",
+		printf("%d %d %s %s %s\n\n",
 			   cluster->remoteNodeCount + 1,
+			   cluster->memberRemoteNodeCount + 1,
 			   cluster->escalated ? "YES" : "NO",
-			   cluster->masterNodeName,
-			   cluster->masterHostName);
+			   cluster->leaderNodeName,
+			   cluster->leaderHostName);
 
 		for (i = 0; i < cluster->nodeCount; i++)
 		{
 			PCPWDNodeInfo *watchdog_info = &cluster->nodeList[i];
 
-			printf("%s %s %d %d %d %s\n",
+			printf("%s %s %d %d %d %s %d %s\n",
 				   watchdog_info->nodeName,
 				   watchdog_info->hostName,
 				   watchdog_info->pgpool_port,
 				   watchdog_info->wd_port,
 				   watchdog_info->state,
-				   watchdog_info->stateName);
+				   watchdog_info->stateName,
+				   watchdog_info->membership_status,
+				   watchdog_info->membership_status_string);
 		}
 	}
 }
@@ -700,11 +833,17 @@ app_require_nodeID(void)
 {
 	return (current_app_type->app_type == PCP_ATTACH_NODE ||
 			current_app_type->app_type == PCP_DETACH_NODE ||
-			current_app_type->app_type == PCP_NODE_INFO ||
+			current_app_type->app_type == PCP_HEALTH_CHECK_STATS ||
 			current_app_type->app_type == PCP_PROMOTE_NODE ||
 			current_app_type->app_type == PCP_RECOVERY_NODE);
 }
 
+static inline bool
+app_support_cluster_mode(void)
+{
+	return (current_app_type->app_type == PCP_STOP_PGPOOL ||
+			current_app_type->app_type == PCP_RELOAD_CONFIG);
+}
 
 static void
 usage(void)
@@ -735,14 +874,32 @@ usage(void)
 		fprintf(stderr, "  -n, --node-id=NODEID   ID of a backend node\n");
 	}
 
+	if (current_app_type->app_type == PCP_NODE_INFO)
+	{
+		fprintf(stderr, "  -n, --node-id=NODEID   ID of a backend node\n");
+		fprintf(stderr, "  -a, --all              display all backend nodes information\n");
+	}
+
 	if (current_app_type->app_type == PCP_STOP_PGPOOL)
 	{
 		fprintf(stderr, "  -m, --mode=MODE        MODE can be \"smart\", \"fast\", or \"immediate\"\n");
 	}
+
+	if (app_support_cluster_mode())
+	{
+		fprintf(stderr, "  -s, --scope=SCOPE      SCOPE can be \"cluster\", or \"local\"\n");
+		fprintf(stderr, "                         cluster scope do operations on all Pgpool-II nodes\n");
+		fprintf(stderr, "                         part of the watchdog cluster\n");
+	}
 	if (current_app_type->app_type == PCP_PROMOTE_NODE ||
 		current_app_type->app_type == PCP_DETACH_NODE)
 	{
-		fprintf(stderr, "  -g, --gracefully       promote gracefully(optional)\n");
+		fprintf(stderr, "  -g, --gracefully       promote gracefully (optional)\n");
+	}
+
+	if (current_app_type->app_type == PCP_PROMOTE_NODE)
+	{
+		fprintf(stderr, "  -s, --switchover       switchover primary to specified node (optional)\n");
 	}
 
 	if (current_app_type->app_type == PCP_WATCHDOG_INFO)
@@ -837,9 +994,9 @@ backend_status_to_string(BackendInfo * bi)
 char *
 role_to_str(SERVER_ROLE role)
 {
-	static char *role_str[] = {"master", "slave", "primary", "standby"};
+	static char *role_str[] = {"main", "replica", "primary", "standby"};
 
-	if (role < ROLE_MASTER || role > ROLE_STANDBY)
+	if (role < ROLE_MAIN || role > ROLE_STANDBY)
 		return "unknown";
 	return role_str[role];
 }
@@ -849,7 +1006,7 @@ role_to_str(SERVER_ROLE role)
  *
  * titles: title string array
  * types:  printf format type string array (example: "d")
- * ntitles: size of the arrary
+ * ntitles: size of the array
  */
 static char *
 format_titles(const char **titles, const char **types, int ntitles)
@@ -876,5 +1033,6 @@ format_titles(const char **titles, const char **types, int ntitles)
 		strncat(formatbuf, buf2, sizeof(formatbuf) - strlen(formatbuf) - 2);
 		strcat(formatbuf, "\n");
 	}
+	strcat(formatbuf, "\n");
 	return formatbuf;
 }
