@@ -1,5 +1,6 @@
 /*
- * Copyright (c) 2017-2019	Tatsuo Ishii
+ * Copyright (c) 2017-2018	Tatsuo Ishii
+ * Copyright (c) 2018-2021	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -191,7 +192,7 @@ openfile(char *filename)
 }
 
 /*
- * Connect to the specifed PostgreSQL. If failed, do not return and exit
+ * Connect to the specified PostgreSQL. If failed, do not return and exit
  * within this function.
  */
 static PGconn *
@@ -200,6 +201,7 @@ connect_db(char *host, char *port, char *user, char *database)
 	char		conninfo[1024];
 	PGconn	   *conn;
 	size_t		n;
+	char	*app_name_str = " application_name=pgproto";
 
 	conninfo[0] = '\0';
 	n = sizeof(conninfo);
@@ -225,7 +227,7 @@ connect_db(char *host, char *port, char *user, char *database)
 		n -= sizeof("user=");
 		strncat(conninfo, " user=", n);
 		n -= strlen(user) + 1;
-		strcat(conninfo, user);
+		strncat(conninfo, user, n);
 	}
 
 	if (database && database[0] != '\0')
@@ -236,6 +238,9 @@ connect_db(char *host, char *port, char *user, char *database)
 		strncat(conninfo, database, n);
 	}
 
+	n -= strlen(app_name_str);
+	strncat(conninfo, app_name_str, n);
+
 	conn = PQconnectdb(conninfo);
 
 	if (conn == NULL || PQstatus(conn) == CONNECTION_BAD)
@@ -460,7 +465,7 @@ process_message_type(int kind, char *buf, PGconn *conn)
 }
 
 /*
- * Process function call messaage
+ * Process function call message
  */
 static void
 process_function_call(char *buf, PGconn *conn)
